/***************************************************************************
                          addlinkdialog.cpp  -  K Desktop Planetarium
                             -------------------
    begin                : Sun Oct 21 2001
    copyright            : (C) 2001 by Jason Harris
    email                : kstars@30doradus.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qapplication.h>
#include <qurl.h>
#include <qmessagebox.h>

#include <qlayout.h>
#include <qlabel.h>
#include <qframe.h>
#include <qstring.h>
#include <qbuttongroup.h>
#include <qpushbutton.h>
#include <qpe/mimetype.h>
#include <qpe/applnk.h>

#include "skymap.h"
#include "addlinkdialog.h"

AddLinkDialog::AddLinkDialog( QWidget *parent )
	: QDialog( parent, tr( "Add Custom URL" ), TRUE ) {

	SkyMap *map = (SkyMap *)parent;

//Create widgets and layout managers
	vlay = new QVBoxLayout( this, 2, 2 );
	hlayTitle = new QHBoxLayout();
	hlayURL = new QHBoxLayout();
	hlayBrowse = new QHBoxLayout();

	TypeGroup = new QButtonGroup( 1, Qt::Vertical, tr( "Resource Type" ), this );
	ImageRadio = new QRadioButton( tr( "Image" ), TypeGroup );
	ImageRadio->setChecked( true );
	InfoRadio = new QRadioButton( tr( "Information" ), TypeGroup );
	InfoRadio->setChecked( false );

	QLabel *titleLabel = new QLabel( this, "TitleLabel" );
	titleLabel->setText( tr( "Menu Text: " ) );
	TitleEntry = new QLineEdit( tr( "Show image of " ) + map->clickedObject()->name(), this );

	QLabel *URLLabel = new QLabel( this, "URLLabel" );
	URLLabel->setText( tr( "URL: " ) );
	URLEntry = new QLineEdit( "http://", this );

	BrowserButton = new QPushButton( tr( "Check URL" ), this );

	//pack widgets into layout managers
	hlayTitle->addWidget( titleLabel );
	hlayTitle->addWidget( TitleEntry );
	hlayURL->addWidget( URLLabel );
	hlayURL->addWidget( URLEntry );
	hlayBrowse->addStretch();
	hlayBrowse->addWidget( BrowserButton );

	vlay->addWidget( TypeGroup );
	vlay->addLayout( hlayTitle );
	vlay->addLayout( hlayURL );
	vlay->addLayout( hlayBrowse );
	vlay->addStretch();
	vlay->activate();

//connect signals to slots
	connect( BrowserButton, SIGNAL( clicked() ), this, SLOT( checkURL() ) );
	connect( TypeGroup, SIGNAL( clicked( int ) ), this, SLOT( changeDefaultTitle( int ) ) );
}

void AddLinkDialog::checkURL( void ) {
	QUrl url ( URLEntry->text() );
	if (url.isValid()) {   //Is the string a valid URL?
		MimeType mt( "text/html" );
		const AppLnk* app = mt.application();
		if ( app ) {
			app->execute( url.toString( TRUE ) );   //If so, launch the browser to see if it's the correct document
		}
	} else {   //If not, print a warning message box that offers to open the browser to a search engine.
		QString message = tr( "The URL is not valid.  Would you like me to open a browser window\nto the Google search engine?" );
		QString caption = tr( "Invalid URL" );
		if ( QMessageBox::warning( 0, caption, message, QMessageBox::Yes, QMessageBox::Yes )==QMessageBox::Yes ) {
			MimeType mt( "text/html" );
			const AppLnk* app = mt.application();
			if ( app ) {
				app->execute( "http://www.google.com" );
			}
		}
	}
}

void AddLinkDialog::changeDefaultTitle( int id ) {
	SkyMap *map = (SkyMap *)parent();

//If the user hasn't changed the default title text, but the link type (image/webpage)
//has been toggled, update the default title text
	if ( id==1 && TitleEntry->text().startsWith( tr( "Show image of " ) ) ) {
		TitleEntry->setText( tr( "Show webpage about " ) + map->clickedObject()->name() );
	}

	if ( id==0 && TitleEntry->text().startsWith( tr( "Show webpage about " ) ) ) {
		TitleEntry->setText( tr( "Show image of " ) + map->clickedObject()->name() );
	}
}
