/***************************************************************************
                          addlocationdialog.cpp  -  K Desktop Planetarium
                             -------------------
    begin                : Sun Feb 11 2001
    copyright            : (C) 2001 by Jason Harris
    email                : jharris@30doradus.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <qmessagebox.h>
#include <qpe/global.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qlistbox.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qgroupbox.h>
#include <qvalidator.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qfile.h>
#include <qtextstream.h>
#include <stdlib.h>
#include "kstars.h"
#include <stdio.h>

#include "addlocationdialog.h"

AddLocationDialog::AddLocationDialog( QWidget* parent )
    : QDialog( parent, tr( "Set Location" ), TRUE ) {

//Create Layout managers
	glay2 = new QGridLayout( this, 5, 2, 2, 2 ); //this layout will be added to vlay2

//Create widgets
	NewCityLabel = new QLabel( this );
	NewProvinceLabel = new QLabel( this );
	NewCountryLabel = new QLabel( this );
	LongLabel = new QLabel( this );
	LatLabel = new QLabel( this );

	NewCityName = new QLineEdit( this );
	NewProvinceName = new QLineEdit( this );
	NewCountryName = new QLineEdit( this );
	NewLong = new QLineEdit( this );
	NewLat = new QLineEdit( this );
	
	NewCityLabel->setText( tr( "City:" ) );
	NewProvinceLabel->setText( tr( "State/Province:" ) );
	NewCountryLabel->setText( tr( "Country:" ) );
	LongLabel->setText( tr( "Longitude:" ) );
	LatLabel->setText( tr( "Latitude:" ) );
	
//Pack the widgets into the layouts
	glay2->addWidget( NewCityLabel, 0, 0 );
	glay2->addWidget( NewProvinceLabel, 1, 0 );
	glay2->addWidget( NewCountryLabel, 2, 0 );
	glay2->addWidget( LongLabel, 3, 0 );
	glay2->addWidget( LatLabel, 4, 0 );
	glay2->addWidget( NewCityName, 0, 1 );
	glay2->addWidget( NewProvinceName, 1, 1 );
	glay2->addWidget( NewCountryName, 2, 1 );
	glay2->addWidget( NewLong, 3, 1 );
	glay2->addWidget( NewLat, 4, 1 );

	glay2->setRowStretch( 5, 1 );

	glay2->activate();
	
	NewLong->setValidator( new QDoubleValidator( -180.0, 180.0, 4, NewLong ) );
	NewLat->setValidator( new QDoubleValidator( -90.0, 90.0, 4, NewLat ) );

  connect( NewLong, SIGNAL( textChanged( const QString & ) ), this, SLOT( checkLong() ) );
  connect( NewLat, SIGNAL( textChanged( const QString & ) ), this, SLOT( checkLat() ) );
}

AddLocationDialog::~AddLocationDialog(){
}

void AddLocationDialog::accept( void ) {
	if ( NewCityName->text().isEmpty() || NewCountryName->text().isEmpty() ||
		NewLat->text().isEmpty() || NewLong->text().isEmpty() ) {

		QString message = tr( "All fields (except Province) must be filled to add this location." );		
		QMessageBox::warning( 0, tr( "Fields are empty" ), message );
		exec();
	} else {
		QDialog::accept();
	}
}

void AddLocationDialog::checkLong( void ) {
	double lng = NewLong->text().toDouble();
	if ( lng < -180.0 || lng > 180.0 ) {
		QString message = tr( "The longitude must be expressed as \na floating-point number between -180.0 and 180.0" );
		QMessageBox::warning( 0, tr( "Invalid Longitude" ), message );
		NewLong->clear();
	}
}

void AddLocationDialog::checkLat( void ) {
	double lat = NewLat->text().toDouble();
	if ( lat < -90.0 || lat > 90.0 ) {
		QString message = tr( "The latitude must be expressed as \na floating-point number between -90.0 and 90.0" );
		QMessageBox::warning( 0, tr( "Invalid Latitude" ), message );
		NewLat->clear();
	}
}

