/***************************************************************************
                          finddialog.cpp  -  K Desktop Planetarium
                             -------------------
    begin                : Wed Jul 4 2001
    copyright            : (C) 2001 by Jason Harris
    email                : jharris@30doradus.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qframe.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qlist.h>
#include <qlistbox.h>
#include <qcombobox.h>
#include <qpushbutton.h>

#include "finddialog.h"
#include "kstars.h"

FindDialog::FindDialog( QWidget* parent )
	: QDialog( parent, tr( "Find Object" ), TRUE ) {
	
//Create Layout managers
	vlay = new QVBoxLayout( this, 2, 2 );
	hlay = new QHBoxLayout( 2 ); //this mgr will be added to vlay

//Create Widgets
	SearchBox = new QLineEdit( this, "SearchBox" );
	SearchBox->setFocus();
	
	filterTypeLabel = new QLabel( this, "filterTypeLabel" );
	filterTypeLabel->setAlignment( AlignRight | AlignVCenter );
	filterTypeLabel->setText( tr( "Filter by type: " ) );
	
	filterType = new QComboBox( this, "filterType" );
	filterType->setEditable( false );
	filterType->insertItem( tr ("Any") );
	filterType->insertItem( tr ("Constellations") );
	filterType->insertItem( tr ("Stars") );
	filterType->insertItem( tr ("Double Stars") );
	filterType->insertItem( tr ("Planets") );
	filterType->insertItem( tr ("Open Clusters") );
	filterType->insertItem( tr ("Glob. Clusters") );
	filterType->insertItem( tr ("Gas. Nebulae") );
	filterType->insertItem( tr ("Plan. Nebulae") );
	filterType->insertItem( tr ("SN Remnants") );
	filterType->insertItem( tr ("Galaxies") );
	
	SearchList = new QListBox( this, "SearchList" );
	SearchList->setVScrollBarMode( QListBox::AlwaysOn );
	SearchList->setHScrollBarMode( QListBox::AlwaysOff );

//Pack Widgets into layout manager
	hlay->addWidget( filterTypeLabel, 0, 0 );
	hlay->addWidget( filterType, 0, 0 );
	
	vlay->addWidget( SearchBox, 0, 0 );
	vlay->addWidget( SearchList, 0, 0 );
	vlay->addLayout( hlay, 0 );
	
	vlay->activate();

	currentitem = 0;
	
//Connect signals to slots
	connect( SearchBox, SIGNAL( textChanged( const QString & ) ), SLOT( filter() ) );
	connect( filterType, SIGNAL( activated( int ) ), this, SLOT( filterByType( int ) ) );
	connect (SearchList, SIGNAL (selectionChanged  (QListBoxItem *)), SLOT (updateSelection (QListBoxItem *)));
	
	
	startTimer (0);		// we use this timer to make sure initObjectList is called only after
                   // the dialog is created and shown.
}

FindDialog::~FindDialog() {
	delete SearchList;
}

void FindDialog::timerEvent (QTimerEvent *)
{
	killTimers(); //make sure timer fires only one time.
	initObjectList();

}

void FindDialog::initObjectList( void ) {
  KStars *p = (KStars *)parent();

	//Fill the SearchBox with the full list of object name strings
	SearchBox->clear();
	SearchList->sort (false);		// the Items will be faster inserted
	int i = 0;
	
	for (SkyObjectName *name = p->data()->ObjNames->first(); name; name = p->data()->ObjNames->next()) {
		new SkyObjectNameListItem ( SearchList, name, p->options()->useLatinConstellNames );
		if (i++ >= 5000) {            // Every 5000 name insertions,
			qApp->processEvents (50);		// spend 50 msec processing KApplication events
			i = 0;
		}
	}
	SearchList->sort (true);	// sort alphabetically, once all names have been added
	setListItemEnabled();  // Automatically highlight first item
}

void FindDialog::filter() {  //Filter the list of names with the string in the SearchBox
  KStars *p = (KStars *)parent();
	
	int i=0;
	SearchList->clear();
	filterType->setCurrentItem( 0 );
		
	//Loop through the full objList.  Add those objects which start
	//with the text in the SearchBox to the filteredList.
	//Similarly, add objNames items to SearchList.
	QString searchFor = SearchBox->text().lower();
	if ( p->options()->useLatinConstellNames ) {	// check if latin names are used
		for ( SkyObjectName *name = p->data()->ObjNames->first(); name; name = p->data()->ObjNames->next() ) {
			if ( name->text().lower().startsWith( searchFor ) ) {
				new SkyObjectNameListItem ( SearchList, name, true ); // p->options()->useLatinConstellNames );
				if (i++ >= 5000) {              //Every 5000 name insertions,
					qApp->processEvents ( 50 );		//spend 50 msec processing KApplication events
					i = 0;
				}
			}
			setListItemEnabled(); // Automatically highlight first item
		}
	}
	else {	// if localized names are used
		for ( SkyObjectName *name = p->data()->ObjNames->first(); name; name = p->data()->ObjNames->next() ) {
			if ( name->translatedText().lower().startsWith( searchFor ) ) {
				new SkyObjectNameListItem ( SearchList, name, false ); // p->options()->useLatinConstellNames );
				if (i++ >= 5000) {             //Every 5000 name insertions,
					qApp->processEvents ( 50 );  //spend 50 msec processing KApplication events
					i = 0;
				}
			}
		setListItemEnabled(); // Automatically highlight first item
		}		
	}
}

void FindDialog::filterByType( int iType ) {
	KStars *p = (KStars *)parent();

	int i = 0;	
	SearchList->clear();	// QListBox
	SearchBox->clear();	// QLineEdit

	//Construct filteredList; select by object type
	switch ( iType ) {
		case 0: //Any
			for (SkyObjectName *name = p->data()->ObjNames->first(); name; name = p->data()->ObjNames->next()) {
				new SkyObjectNameListItem ( SearchList, name, p->options()->useLatinConstellNames );
				if (i++ >= 5000) {           //Every 5000 name insertions,
					qApp->processEvents (50);  //spend 50 msec processing KApplication events
					i = 0;
				}
			}
			SearchList->sort (true);
			setListItemEnabled();    // Automatically highlight first item
			break;
		
		default: //All others, filter according to objType
			for (SkyObjectName *name = p->data()->ObjNames->first(); name; name = p->data()->ObjNames->next()) {
				if (name->skyObject()->type() + 2 == iType) {
					new SkyObjectNameListItem ( SearchList, name, p->options()->useLatinConstellNames );
				}
				if (i++ >= 5000) {            //Every 5000 name insertions,
					qApp->processEvents (50);   //spend 50 msec processing KApplication events
					i = 0;
				}
			}
			SearchList->sort (true);
			setListItemEnabled();    // Automatically highlight first item
			break;
	}
}

void FindDialog::setListItemEnabled() {
	SearchList->setSelected (0, true);
	if (!SearchList->isSelected (0))
		updateSelection (0);
}

void FindDialog::updateSelection (QListBoxItem *it) {
	currentitem = (SkyObjectNameListItem *) it;
}
