/***************************************************************************
                          kstars.cpp  -  K Desktop Planetarium
                             -------------------
    begin                : Mon Feb  5 01:11:45 PST 2001
    copyright            : (C) 2001 by Jason Harris
    email                : jharris@30doradus.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
//JH 24.08.2001: reorganized infoPanel
//JH 25.08.2001: added toolbar, converted menu items to KAction objects
//JH 25.08.2001: main window now resizable, window size saved in config file

#include <qfont.h>
#include <qpopupmenu.h>
#include <qtextstream.h>
#include <qlineedit.h>
#include <qsizepolicy.h>
#include <qtooltip.h>
#include <qlayout.h>

#include <qapplication.h>
#include <qpe/config.h>
#include <qaccel.h>
#include <qpe/resource.h>
#include <qmessagebox.h>
#include <qstatusbar.h>
#include <qpopupmenu.h>
#include <qtoolbar.h>
#include <qpe/mimetype.h>
#include <qpe/applnk.h>

#include <stdio.h>
#include <stdlib.h>
#include <stream.h>

#include "timedialog.h"
#include "locationdialog.h"
#include "finddialog.h"
#include "viewopsdialog.h"

#include "kstars.h"

KStars::KStars( KStarsData* kstarsData )
	: QMainWindow( NULL, NULL )
{
// need to set the mainwidget here, in main() this will cause a segfault because
// skymap will created in this constructor and needs kapp->mainWidget()
	qApp->setMainWidget( this );
	// here we get the preloaded data (stars, constellations etc.)

	this->kstarsData = kstarsData;

	//resize( 640, 600 );
	setToolBarsMovable( FALSE );
	initMenuBar();
	initToolBar();
	initStatusBar();
	initOptions();
	initLocation();

// create the widgets
	QWidget *centralWidget = new QWidget( this );
	setCentralWidget( centralWidget );
	infoPanel = new QFrame( centralWidget );
	skymap = new SkyMap( centralWidget );
	skymap->QWidget::setFocus();		// get focus of keyboard and mouse actions (for example zoom in with +)
	skymap->setMinimumSize( 380, 250 );
	skymap->setSizePolicy( QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding ) );


// create the layout of the central widget
	QVBoxLayout *topLayout = new QVBoxLayout( centralWidget );
	topLayout->addWidget( infoPanel );
	topLayout->addWidget( skymap );

/*	//time settings that we couldn't do in KStarsSplash:
	data()->UTime = data()->now.addSecs( int(geo()->TZ()*-3600) );
	data()->LST   = UTtoLST( data()->UTime, geo()->lng() );
	data()->LSTh.setH( data()->LST.hour(), data()->LST.minute(), data()->LST.second() );

	data()->CurrentDate = data()->getJD( data()->UTime );
	data()->LastSkyUpdate = data()->CurrentDate;
	updateEpoch( data()->CurrentDate );
*/

	//Create the infoPanel, which displays time/date/location data
	infoPanel->setFrameShape( QFrame::Panel );

	//white-on-black color scheme for infoPanel
	QPalette pal( infoPanel->palette() );
	pal.setColor( QPalette::Normal, QColorGroup::Background, QColor( 0, 0, 0 ) );
	pal.setColor( QPalette::Normal, QColorGroup::Foreground, QColor( 255, 255, 255 ) );
	pal.setColor( QPalette::Inactive, QColorGroup::Foreground, QColor( 255, 255, 255 ) );
	infoPanel->setPalette( pal );

	//Create main Layout manager for infoPanel
	iplay = new QHBoxLayout( infoPanel, 2, 2, "iplay" );

	//Create Widgets to appear in the info panel
	LTTime = new QLabel( tr( "LT", "Local Time" ) + ": 00:00:00", infoPanel );
	LTTime->setPalette( pal );

	QFont ipFont = LTTime->font();
	ipFont.setPointSize( ipFont.pointSize() - 2 );

	LTTime->setFont( ipFont );
	LTTime->setFixedSize( LTTime->sizeHint() );

	LTDate = new QLabel( TimeString::shortDate( data()->LTime.date() ), infoPanel );
	LTDate->setPalette( pal );
	LTDate->setFont( ipFont );

	FocusRA = new QLabel( tr( "RA", "Right Ascension" ) + ": 00:00:00    ", infoPanel );
	FocusRA->setPalette( pal );
	FocusRA->setFont( ipFont );
	FocusRA->setFixedSize( FocusRA->sizeHint() );

	FocusDec = new QLabel( tr( "Dec", "Declination" ) + ": +00:00:00", infoPanel );
	FocusDec->setPalette( pal );
	FocusDec->setFont( ipFont );

	FocusAz = new QLabel( tr( "Az", "Azimuth" ) + ": +000:00:00 ", infoPanel );
	FocusAz->setPalette( pal );
	FocusAz->setFont( ipFont );
	FocusAz->setFixedSize( FocusAz->sizeHint() );

	FocusAlt = new QLabel( tr( "Alt", "Altitude" ) + ": +00:00:00", infoPanel );
	FocusAlt->setPalette( pal );
	FocusAlt->setFont( ipFont );

	//create layouts to be added to iplay
	raazlay = new QVBoxLayout( iplay, 0, "raazlay" );
	decaltlay = new QVBoxLayout( iplay, 0, "decaltlay" );
	iplay->addStretch();
	timelay = new QVBoxLayout( iplay, 0, "timelay" );

	//Pack widgets into infoPanel
	raazlay->addWidget( FocusRA );
	raazlay->addWidget( FocusAz );

	decaltlay->addWidget( FocusDec );
	decaltlay->addWidget( FocusAlt );

	timelay->addWidget( LTTime );
	timelay->addWidget( LTDate );

	infoPanel->setFixedHeight( infoPanel->sizeHint().height() );	// use sizeHint() to get accurate size
	resize( options()->windowWidth, options()->windowHeight );

	//Start the clock; intitialize time to system clock
	data()->SysJD_Mark = data()->getJD( QDateTime::currentDateTime() );
	data()->SkyJD_Mark = data()->SysJD_Mark;
	
	tmr = new QTimer( this );
	tmr->start( TIMER_INTERVAL, FALSE );
	QObject::connect( tmr, SIGNAL( timeout() ), this, SLOT( updateTime() ) );

//Try updateTime() and updateEpoch() here to initialize CurrentDate and CurrentEpoch...
	data()->CurrentEpoch = 0.0; //guarantee that updateTime() calls updateEpoch()
	updateTime();

	initAltAz();

	SkyPoint newPoint( options()->focusRA, options()->focusDec );

//Set focus of Skymap.
//if user was tracking last time, track on same object now.
	if ( options()->isTracking ) {

    //Set default position in case focus object is below horizon
		SkyPoint DefaultFocus;
		DefaultFocus.setAz( 180.0 );
		DefaultFocus.setAlt( 45.0 );
		DefaultFocus.HorizontalToEquatorial( data()->LSTh, geo()->lat() );
		skymap->setFocus( &DefaultFocus );

		if ( (options()->focusObject== tr( "star" ) ) ||
		     (options()->focusObject== tr( "nothing" ) ) ) {
			skymap->setClickedPoint( &newPoint );
		} else {
			skymap->setClickedObject( getObjectNamed( options()->focusObject ) );
			if ( skymap->clickedObject() ) {
				skymap->setClickedPoint( skymap->clickedObject()->pos() );
			} else {
				skymap->setClickedPoint( &newPoint );
			}
		}
	} else {
		skymap->setClickedPoint( &newPoint );
	}

	skymap->slotCenter();

	data()->HourAngle.setH( data()->LSTh.Hours() - skymap->focus()->ra().Hours() );

	skymap->setOldFocus( skymap->focus() );
	skymap->oldfocus()->setAz( skymap->focus()->az() );
	skymap->oldfocus()->setAlt( skymap->focus()->alt() );
}

KStars::~KStars()
{
/*
	//Sync the config file
	Config config( "KStars" );
	config.setGroup( "View");
	config.setGroup( "Location" );
	config.writeEntry( "City", options()->CityName );
	config.writeEntry( "Province", options()->ProvinceName );
	config.writeEntry( "Country", options()->CountryName );
	config.setGroup( "View" );
	config.writeEntry( "SkyColor", 	options()->colorSky );
	config.writeEntry( "MWColor", 		options()->colorMW );
	config.writeEntry( "EclColor", 		options()->colorEcl );
	config.writeEntry( "EqColor", 		options()->colorEq );
	config.writeEntry( "HorzColor", 	options()->colorHorz );
	config.writeEntry( "GridColor", 	options()->colorGrid );
	config.writeEntry( "MessColor", 	options()->colorMess );
	config.writeEntry( "NGCColor", 	options()->colorNGC );
	config.writeEntry( "ICColor", 		options()->colorIC );
	config.writeEntry( "CLineColor", options()->colorCLine );
	config.writeEntry( "CNameColor", options()->colorCName );
	config.writeEntry( "SNameColor", options()->colorSName );
	config.writeEntry( "HSTColor", 	options()->colorHST );
	config.writeEntry( "StarColorMode", options()->starColorMode );
	config.writeEntry( "StarColorsIntensity", options()->starColorIntensity );
	config.writeEntry( "ShowBSC", 		options()->drawBSC );
	config.writeEntry( "ShowMess", 	options()->drawMessier );
	config.writeEntry( "ShowMessImages", 	options()->drawMessImages );
	config.writeEntry( "ShowNGC", 		options()->drawNGC );
	config.writeEntry( "ShowIC", 		options()->drawIC );
	config.writeEntry( "ShowCLines", options()->drawConstellLines );
	config.writeEntry( "ShowCNames", options()->drawConstellNames );
	config.writeEntry( "UseLatinConstellationNames", options()->useLatinConstellNames );
	config.writeEntry( "UseLocalConstellationNames", options()->useLocalConstellNames );
	config.writeEntry( "UseAbbrevConstellationNames", options()->useAbbrevConstellNames );
	config.writeEntry( "ShowMilkyWay", options()->drawMilkyWay );
	config.writeEntry( "ShowGrid", options()->drawGrid );
	config.writeEntry( "ShowEquator", options()->drawEquator );
	config.writeEntry( "ShowEcliptic", options()->drawEcliptic );
	config.writeEntry( "ShowHorizon", options()->drawHorizon );
	config.writeEntry( "ShowGround", options()->drawGround );
	config.writeEntry( "ShowSun", 		options()->drawSun );
	config.writeEntry( "ShowMoon", 	options()->drawMoon );
	config.writeEntry( "ShowMercury", 	options()->drawMercury );
	config.writeEntry( "ShowVenus", 	options()->drawVenus );
	config.writeEntry( "ShowMars", 	options()->drawMars );
	config.writeEntry( "ShowJupiter", 	options()->drawJupiter );
	config.writeEntry( "ShowSaturn", 	options()->drawSaturn );
	config.writeEntry( "ShowUranus", 	options()->drawUranus );
	config.writeEntry( "ShowNeptune", 	options()->drawNeptune );
	config.writeEntry( "ShowPluto", 	options()->drawPluto );
	config.writeEntry( "IsTracking", 	options()->isTracking );
	if ( skymap->foundObject() != NULL ) {
		config.writeEntry( "FocusObject",  skymap->foundObject()->name() );
	} else {
		config.writeEntry( "FocusObject", tr( "nothing" ) );
	}
	config.writeEntry( "UseAltAz", 	options()->useAltAz );
        QString snum;
	config.writeEntry( "FocusRA", snum.setNum( skymap->focus()->ra().Hours() ) );
	config.writeEntry( "FocusDec", snum.setNum( skymap->focus()->dec().Degrees() ) );
	config.writeEntry( "ZoomLevel", data()->ZoomLevel );
	config.writeEntry( "windowWidth", width() );
	config.writeEntry( "windowHeight", height() );
	config.writeEntry( "magLimitDrawStar", 	 snum.setNum( options()->magLimitDrawStar ) );
	config.writeEntry( "magLimitDrawStarInfo",snum.setNum( options()->magLimitDrawStarInfo ) );
	config.writeEntry( "drawStarName", 			 options()->drawStarName );
	config.writeEntry( "drawStarMagnitude",   options()->drawStarMagnitude );
*/

	// remove data object
	delete kstarsData;
	//kstarsData = 0;

	delete Location;
	delete LTTime;
	delete LTDate;

	delete skymap;
}

void KStars::initMenuBar() {
	QMenuBar *menuBar = new QMenuBar( this );

	QPopupMenu *p;

	p = new QPopupMenu;
        actTimeNow = new QAction( tr( "Set Time to Now" ), QString::null, 0, this);
	connect( actTimeNow, SIGNAL( activated() ), this, SLOT( mSetTimeToNow() ) );
        actTimeSet = new QAction( tr( "Set Time" ), Resource::loadIconSet( "kstars/clock" ), tr( "Set Time..." ), 0, this);
	connect( actTimeSet, SIGNAL( activated() ), this, SLOT( mSetTime() ) );
        actTimeRun = new QAction( tr( "Stop Clock" ), Resource::loadIconSet( "kstars/player_pause" ), QString::null, 0, this);
	connect( actTimeRun, SIGNAL( activated() ), this, SLOT( mToggleTimer() ) );
	actTimeNow->addTo( p );
	actTimeSet->addTo( p );
	actTimeRun->addTo( p );
	menuBar->insertItem( tr( "Time" ), p );

	p = new QPopupMenu;
	p->insertItem( tr( "Zenith" ), this, SLOT( mZenith() ));
        actFind = new QAction( tr( "Find Object" ), Resource::loadIconSet( "kstars/find" ), tr( "Find Object..." ), 0, this);
	connect( actFind, SIGNAL( activated() ), this, SLOT( mFind() ) );
	actFind->addTo( p );
	actTrack = new QAction( tr( "Track Object" ), Resource::loadIconSet( "kstars/unlock" ), QString::null, 0, this);
	connect( actTrack, SIGNAL( activated() ), this, SLOT( mTrack() ) );
	actTrack->addTo( p );
	p->insertSeparator();

	actLocation = new QAction( tr( "Geographic Location" ), Resource::loadIconSet( "kstars/earth" ), tr( "Geographic..." ), 0, this);
	connect( actLocation, SIGNAL( activated() ), this, SLOT( mGeoLocator() ) );
	actLocation->addTo( p );
	menuBar->insertItem( tr( "Location" ), p );

	p = new QPopupMenu;
	actZoomIn = new QAction( tr( "Zoom In" ), Resource::loadIconSet( "kstars/viewmag+" ), QString::null, 0, this);
	connect( actZoomIn, SIGNAL( activated() ), this, SLOT( mZoomIn() ) );
	actZoomOut = new QAction( tr( "Zoom Out" ), Resource::loadIconSet( "kstars/viewmag-" ), QString::null, 0, this);
	connect( actZoomOut, SIGNAL( activated() ), this, SLOT( mZoomOut() ) );
	actZoomIn->addTo( p );
	actZoomOut->addTo( p );

	p->insertSeparator();
	actViewOps = new QAction( tr( "Configure" ), Resource::loadIconSet( "kstars/configure" ), tr( "Configure..." ), 0, this);
	connect( actViewOps, SIGNAL( activated() ), this, SLOT( mViewOps() ) );
	actViewOps->addTo( p );
	menuBar->insertItem( tr( "View" ), p );
}

void KStars::initToolBar() {
	QToolBar *toolBar = new QToolBar( this );

	actZoomIn->addTo( toolBar );
	actZoomOut->addTo( toolBar );
	actTrack->addTo( toolBar );
	actTimeRun->addTo( toolBar );

	TimeStep = new TimeSpinBox( toolBar );
	QToolTip::add( TimeStep, tr( "Time Step (seconds)" ) );
	connect( TimeStep, SIGNAL( valueChanged( int ) ), this, SLOT( changeTimeStep() ) );

	toolBar = new QToolBar( this );
	toolBar->setHorizontalStretchable( TRUE );
}

void KStars::initStatusBar() {
}

/** Menu Slot Functions **/
void KStars::mSetTimeToNow() {
	data()->SysJD_Mark = data()->getJD( QDateTime::currentDateTime() );
	data()->SkyJD_Mark = data()->SysJD_Mark;

//	data()->LTime.setTime( QTime::currentTime() );
//	data()->LTime.setDate( QDate::currentDate() );
//	data()->UTime.setTime( data()->LTime.time() );
//	data()->UTime.setDate( data()->LTime.date() );
//	data()->UTime = data()->UTime.addSecs( int( geo()->TZ()*-3600) );

//	data()->then = QDateTime::currentDateTime();
	updateTime();
}

void KStars::initOptions()
{
	// Get initial Location from config()
	Config config( "KStars" );
	config.setGroup( "Location" );
	options()->CityName = config.readEntry( "City", "Greenwich" );

	if ( config.readEntry( "State", "" ).length() ) { //old version of config file
		options()->ProvinceName = config.readEntry( "State", "" );
		options()->CountryName = config.readEntry( "State", "United Kingdom" );
	} else {
		options()->ProvinceName = config.readEntry( "Province", "" );
		options()->CountryName = config.readEntry( "Country", "United Kingdom" );
	}

	config.setGroup( "View" );
	options()->colorEcl		= config.readEntry( "EclColor", "#663" );
	options()->colorEq 		= config.readEntry( "EqColor", "#FFF" );
	options()->colorMW 		= config.readEntry( "MWColor", "#123" );
	options()->colorSky 		= config.readEntry( "SkyColor", "#002" );
	options()->colorHorz 	= config.readEntry( "HorzColor", "#5A3" );
	options()->colorGrid 	= config.readEntry( "GridColor", "#456" );
	options()->colorMess 	= config.readEntry( "MessColor", "#0F0" );
	options()->colorNGC 		= config.readEntry( "NGCColor", "#066" );
	options()->colorIC 		= config.readEntry( "ICColor", "#439" );
	options()->colorCLine 	= config.readEntry( "CLineColor", "#555" );
	options()->colorCName 	= config.readEntry( "CNameColor", "#AA7" );
	options()->colorSName 	= config.readEntry( "SNameColor", "#7AA" );
	options()->colorHST 		= config.readEntry( "HSTColor", "#A00" );
	options()->drawBSC 		= config.readBoolEntry( "ShowBSC", true );
	options()->drawMessier = config.readBoolEntry( "ShowMess", true );
	options()->drawMessImages = config.readBoolEntry( "ShowMessImages", true );
	options()->drawNGC 		= config.readBoolEntry( "ShowNGC", true );
	options()->drawIC 			= config.readBoolEntry( "ShowIC", true );
	options()->drawConstellLines = config.readBoolEntry( "ShowCLines", true );
	options()->drawConstellNames = config.readBoolEntry( "ShowCNames", true );
	options()->useLatinConstellNames = config.readBoolEntry( "UseLatinConstellationNames", true );
	options()->useLocalConstellNames = config.readBoolEntry( "UseLocalConstellationNames", false );
	options()->useAbbrevConstellNames = config.readBoolEntry( "UseAbbrevConstellationNames", false );
	options()->drawMilkyWay = config.readBoolEntry( "ShowMilkyWay", true );
	options()->drawGrid = config.readBoolEntry( "ShowGrid", true );
	options()->drawEquator = config.readBoolEntry( "ShowEquator", true );
	options()->drawEcliptic = config.readBoolEntry( "ShowEcliptic", true );
	options()->drawHorizon = config.readBoolEntry( "ShowHorizon", true );
	options()->drawGround = config.readBoolEntry( "ShowGround", true );
	options()->drawSun = config.readBoolEntry( "ShowSun", true );
	options()->drawMoon = config.readBoolEntry( "ShowMoon", true );
	options()->drawMercury = config.readBoolEntry( "ShowMercury", true );
	options()->drawVenus = config.readBoolEntry( "ShowVenus", true );
	options()->drawMars = config.readBoolEntry( "ShowMars", true );
	options()->drawJupiter = config.readBoolEntry( "ShowJupiter", true );
	options()->drawSaturn = config.readBoolEntry( "ShowSaturn", true );
	options()->drawUranus = config.readBoolEntry( "ShowUranus", true );
	options()->drawNeptune = config.readBoolEntry( "ShowNeptune", true );
	options()->drawPluto = config.readBoolEntry( "ShowPluto", true );
	options()->useAltAz = config.readBoolEntry( "UseAltAz", true );
	options()->isTracking = config.readBoolEntry( "IsTracking", false );
	options()->focusObject = config.readEntry( "FocusObject", "nothing" );
	options()->focusDec = config.readEntry( "FocusDec", "45.0" ).toDouble();
	options()->focusRA = config.readEntry( "FocusRA", "180.0" ).toDouble();
//	options()->magLimitDrawStar = config.readEntry( "magLimitDrawStar", "8.0" ).toDouble();		// readed in KStarsOptions()
	options()->magLimitDrawStarInfo = config.readEntry( "magLimitDrawStarInfo", "2.0" ).toDouble();
	options()->drawStarName = config.readBoolEntry( "drawStarName", false );
	options()->drawStarMagnitude = config.readBoolEntry( "drawStarMagnitude", false );
	options()->windowWidth = config.readNumEntry( "windowWidth", 600 );
	options()->windowHeight = config.readNumEntry( "windowHeight", 600 );
	options()->starColorMode = config.readNumEntry( "StarColorMode", 0 );
	options()->starColorIntensity = config.readNumEntry ("StarColorsIntensity", 4);
}

void KStars::initLocation() {
	//Initialize geographic location
	bool bFound = false; bool oldConfig = false;
	GeoLocation *GeoData;

	Config config( "KStars" );
	config.setGroup( "Location" );
	if ( !config.readEntry( "State", "" ).stripWhiteSpace().isEmpty() ) {
		oldConfig = true;
    config.writeEntry( "State", "" ); //ignore this key from now on...
	}

	for (GeoData = data()->geoList.first(); GeoData; GeoData = data()->geoList.next())
	{

	//If the config file is old (has a State key) match a city if the city name
  //AND EITHER the province name OR the country name match
  //(old config files set both province and country to "StateName")
  //this can produce the wrong city, if more than one matches these criteria...
		if ( oldConfig ) {
			if ( (GeoData->name().lower() == options()->CityName.lower()) &&
					( (GeoData->province().lower() == options()->ProvinceName.lower()) ||
					(GeoData->country().lower() == options()->CountryName.lower()) ) )
			{
				bFound = TRUE;
				if ( GeoData->province().lower() != options()->ProvinceName.lower() )
					options()->ProvinceName = GeoData->province();
				if ( GeoData->country().lower() != options()->CountryName.lower() )
					options()->CountryName = GeoData->country();
				break ;
			}
		} else {
	//Otherwise, require all three fields (City, Province, and Country) to match
			if ( options()->ProvinceName.stripWhiteSpace().length() ) {
				if ( (GeoData->name().lower() == options()->CityName.lower()) &&
						(GeoData->province().lower() == options()->ProvinceName.lower()) &&
						(GeoData->country().lower() == options()->CountryName.lower()) )
				{
					bFound = TRUE;
					break ;
				}
			} else {
				if ( (GeoData->name().lower() == options()->CityName.lower()) &&
						(GeoData->country().lower() == options()->CountryName.lower()) )
				{
					bFound = TRUE;
					break ;
				}

			}
		}
	}

	if ( !bFound ) { // set city, province and country to default values
		options()->CityName = "Greenwich";
		options()->ProvinceName = "";
		options()->CountryName = "United Kingdom";
		for (GeoData = data()->geoList.first(); GeoData; GeoData = data()->geoList.next())
		{
			if ( (GeoData->name().lower() == options()->CityName.lower()) &&
					(GeoData->province().lower() == options()->ProvinceName.lower()) &&
					(GeoData->country().lower() == options()->CountryName.lower()) )
			{
				bFound = TRUE;
				break ;
			}
		}
	}

	if (bFound) {
		Location = new GeoLocation (GeoData);
	} else { //couldn't set geographic location, so set the "null" location.
		QString message = tr( "Could not set geographic location!" );
		QMessageBox::warning( 0, tr( "No location set" ), message );
		Location = new GeoLocation();
	}
}

void KStars::initAltAz()
{
	// Recompute Alt, Az coords for all objects. This was not possible
	// within loading the KStarsData object. Do it now.
	// Solar System
	data()->Sun->pos()->EquatorialToHorizontal( data()->LSTh, geo()->lat() );
	data()->Mercury->pos()->EquatorialToHorizontal( data()->LSTh, geo()->lat() );
	data()->Venus->pos()->EquatorialToHorizontal( data()->LSTh, geo()->lat() );
	data()->Mars->pos()->EquatorialToHorizontal( data()->LSTh, geo()->lat() );
	data()->Jupiter->pos()->EquatorialToHorizontal( data()->LSTh, geo()->lat() );
	data()->Saturn->pos()->EquatorialToHorizontal( data()->LSTh, geo()->lat() );
	data()->Uranus->pos()->EquatorialToHorizontal( data()->LSTh, geo()->lat() );
	data()->Neptune->pos()->EquatorialToHorizontal( data()->LSTh, geo()->lat() );
	data()->Pluto->pos()->EquatorialToHorizontal( data()->LSTh, geo()->lat() );

	// Stars
	for (SkyObject *obj = data()->starList.first(); obj; obj = data()->starList.next()) {
		obj->pos()->updateCoords( data()->CurrentEpoch, data()->Obliquity, data()->dObliq, data()->dEcLong );
		obj->pos()->EquatorialToHorizontal( data()->LSTh, geo()->lat() );
	}

	// Messier
	for (SkyObject *obj = data()->messList.first(); obj; obj = data()->messList.next()) {
		obj->pos()->updateCoords( data()->CurrentEpoch, data()->Obliquity, data()->dObliq, data()->dEcLong );
		obj->pos()->EquatorialToHorizontal( data()->LSTh, geo()->lat() );
	}

	// NGC
	for (SkyObject *obj = data()->ngcList.first(); obj; obj = data()->ngcList.next()) {
		obj->pos()->updateCoords( data()->CurrentEpoch, data()->Obliquity, data()->dObliq, data()->dEcLong );
		obj->pos()->EquatorialToHorizontal( data()->LSTh, geo()->lat() );
	}

	// IC
	for (SkyObject *obj = data()->icList.first(); obj; obj = data()->icList.next()) {
		obj->pos()->updateCoords( data()->CurrentEpoch, data()->Obliquity, data()->dObliq, data()->dEcLong );
		obj->pos()->EquatorialToHorizontal( data()->LSTh, geo()->lat() );
	}

	// Milky Way
	for ( unsigned int j=0; j<11; ++j ) {
		for (SkyPoint *obj = data()->MilkyWay[j].first(); obj; obj = data()->MilkyWay[j].next()) {
			obj->updateCoords( data()->CurrentEpoch, data()->Obliquity, data()->dObliq, data()->dEcLong );
			obj->EquatorialToHorizontal( data()->LSTh, geo()->lat() );
		}
	}

	// CLines
	for (SkyPoint *obj = data()->clineList.first(); obj; obj = data()->clineList.next()) {
		obj->updateCoords( data()->CurrentEpoch, data()->Obliquity, data()->dObliq, data()->dEcLong );
		obj->EquatorialToHorizontal( data()->LSTh, geo()->lat() );
	}

	// CNames
	for (SkyObject *obj = data()->cnameList.first(); obj; obj = data()->cnameList.next()) {
		obj->pos()->updateCoords( data()->CurrentEpoch, data()->Obliquity, data()->dObliq, data()->dEcLong );
		obj->pos()->EquatorialToHorizontal( data()->LSTh, geo()->lat() );
	}

	// Define the Celestial Equator
	for ( unsigned int i=0; i<NCIRCLE; ++i ) {
		SkyPoint *o = new SkyPoint( i*24./NCIRCLE, 0.0 );
		o->EquatorialToHorizontal( data()->LSTh, geo()->lat() );
		data()->Equator.append( o );
	}

  // Define the horizon.
  // Use the celestial Equator as a convenient starting point, but instead of RA and Dec,
  // interpret the coordinates as azimuth and altitude, and then convert to RA, dec.
  // The horizon will be redefined whenever the positions of sky objects are updated.
	for (SkyPoint *point = data()->Equator.first(); point; point = data()->Equator.next()) {
		double sinlat, coslat, sindec, cosdec, sinAz, cosAz;
		double HARad;
		dms dec, HA, RA, Az;
		Az = point->ra();
		Az.SinCos( sinAz, cosAz );
		geo()->lat().SinCos( sinlat, coslat );

		dec.setRadians( asin( coslat*cosAz ) );
		dec.SinCos( sindec, cosdec );
   		HARad = acos( -1.0*(sinlat*sindec)/(coslat*cosdec) );
		if ( sinAz > 0.0 ) { HARad = 2.0*PI() - HARad; }
		HA.setRadians( HARad );
		RA = data()->LSTh.Degrees() - HA.Degrees();

		SkyPoint *o = new SkyPoint( RA, dec );
		o->setAlt( 0.0 );
		o->setAz( Az );

		data()->Horizon.append( o );

		//Define the Ecliptic (use the same ListIteration; interpret coordinates as Ecliptic long/lat)
		double ELong, ELat;
		ELong = point->ra().Degrees();
		ELat = 0.0;
		o = new SkyPoint( 0.0, 0.0 );
		o->setEcliptic( ELong, ELat, data()->CurrentDate );
		o->EquatorialToHorizontal( data()->LSTh, geo()->lat() );
		data()->Ecliptic.append( o );
	}
}

void KStars::changeTime( QDate newDate, QTime newTime ) {
	if (tmr->isActive() ) {
		tmr->stop();
		actTimeRun->setIconSet( Resource::loadIconSet( "kstars/1rightarrow" ) );
		actTimeRun->setText( tr( "Start Clock" ) );
		actTimeRun->setToolTip( tr( "Start Clock" ) );
	}

	data()->SysJD_Mark = data()->getJD( QDateTime::currentDateTime() );
	data()->SkyJD_Mark = data()->getJD( QDateTime( newDate, newTime ) );

	//Make sure Moon, planets, and sky objects are updated immediately
	data()->LastMoonUpdate = data()->CurrentDate - 1.0;
	data()->LastPlanetUpdate = data()->CurrentDate - 1.0;
	data()->LastSkyUpdate = data()->CurrentDate - 1.0;

	updateTime();
}

void KStars::mSetTime() {
	TimeDialog timedialog ( data()->LTime, this );

	timedialog.showMaximized();
	if ( timedialog.exec() == QDialog::Accepted ) {
		QTime newTime( timedialog.selectedTime() );
		QDate newDate( timedialog.selectedDate() );

		changeTime(newDate, newTime);
	}
}

void KStars::mToggleTimer() {
	if ( tmr->isActive() ) {
		tmr->stop();
    actTimeRun->setIconSet( Resource::loadIconSet( "kstars/1rightarrow" ) );
		actTimeRun->setText( tr( "Start Clock" ) );
		actTimeRun->setToolTip( tr( "Start Clock" ) );
	} else {
		data()->SysJD_Mark = data()->getJD( QDateTime::currentDateTime() );
		data()->SkyJD_Mark = data()->getJD( data()->LTime );

		tmr->start( TIMER_INTERVAL, FALSE );
		actTimeRun->setIconSet( Resource::loadIconSet( "kstars/player_pause" ) );
		actTimeRun->setText( tr( "Stop Clock" ) );
		actTimeRun->setToolTip( tr( "Stop Clock" ) );
	}
}

void KStars::mZenith() {
	skymap->focus()->setAlt( 90.0 );
	skymap->focus()->HorizontalToEquatorial( data()->LSTh, geo()->lat() );
	data()->HourAngle.setH( data()->LSTh.Hours() - skymap->focus()->ra().Hours() );
	skymap->Update();
}

void KStars::mZoomIn() {
	actZoomOut->setEnabled (true);
	if ( data()->ZoomLevel < MAXZOOMLEVEL ) {
		++data()->ZoomLevel;
		skymap->Update();
	}
	if ( data()->ZoomLevel == MAXZOOMLEVEL )
		actZoomIn->setEnabled (false);
}

void KStars::mZoomOut() {
	actZoomIn->setEnabled (true);
	if ( data()->ZoomLevel > MINZOOMLEVEL ) {
		--data()->ZoomLevel;
		skymap->Update();
	}
	if ( data()->ZoomLevel == MINZOOMLEVEL )
		actZoomOut->setEnabled (false);
}

void KStars::mFind() {
	FindDialog finddialog (this);
	finddialog.showMaximized();
	if ( finddialog.exec() == QDialog::Accepted && finddialog.currentItem() ) {
		skymap->setClickedObject( finddialog.currentItem()->objName()->skyObject() );
		skymap->setClickedPoint( skymap->clickedObject()->pos() );
		skymap->slotCenter();
	}
}

void KStars::mTrack() {
	if ( options()->isTracking ) {
		options()->isTracking = false;
		actTrack->setIconSet( Resource::loadIconSet( "kstars/unlock" ) );
		skymap->setClickedObject( NULL );
		skymap->setFoundObject( NULL );//no longer tracking foundObject
	} else {
		options()->isTracking = true;
		actTrack->setIconSet( Resource::loadIconSet( "kstars/lock" ) );
	}
}

void KStars::mViewOps() {
	// save options for cancel
	data()->saveOptions();

	ViewOpsDialog viewopsdialog (this);
        viewopsdialog.showMaximized();
	// ask for the new options
	if ( viewopsdialog.exec() != QDialog::Accepted ) {
		// cancelled
		data()->restoreOptions();
	}
}

void KStars::mGeoLocator() {
	LocationDialog locationdialog (this);
	locationdialog.showMaximized();
	if ( locationdialog.exec() == QDialog::Accepted ) {
	 	int ii = locationdialog.getCityIndex();
 		if ( ii >= 0 ) {
 			geo()->reset( data()->geoList.at(ii) );
			options()->CityName = geo()->name();
			options()->ProvinceName = geo()->province();
			options()->CountryName = geo()->country();

 			// Adjust Local time for new time zone
	 		data()->LTime.setDate( data()->UTime.date() );
 			data()->LTime.setTime( data()->UTime.time() );
 			data()->LTime = data()->LTime.addSecs( int(geo()->TZ()*3600) );

			data()->LST = UTtoLST( data()->UTime, geo()->lng() );
			data()->LSTh.setH( data()->LST.hour(), data()->LST.minute(), data()->LST.second() );
			data()->HourAngle.setH( data()->LSTh.Hours() - skymap->focus()->ra().Hours() );

      //need to recompute Alt/Az coordinates of all objects, so
			//adjust LastSkyUpdate to ensure computation.  Then
			//explicitly call updateTime()
			data()->LastSkyUpdate -= 1.0; // a full day, should be plenty
			updateTime();
	 	}
 	}
}

void KStars::updateTime( void ) {
	data()->SysJD = data()->getJD( QDateTime::currentDateTime() );
	data()->SysJD_Elapsed = data()->SysJD - data()->SysJD_Mark;

	//advance clock by factor*(time elapsed since system time last marked)
	float factor = TimeStep->text().toFloat();
	data()->LTime = data()->getDateTime( data()->SkyJD_Mark + factor*data()->SysJD_Elapsed );
	data()->UTime = data()->LTime.addSecs( int( -3600*geo()->TZ() ) );

	LTTime->setText( tr( "LT", "Local Time" ) + ": " + data()->LTime.time().toString() );
	LTDate->setText( data()->LTime.date().toString() );

	data()->CurrentDate = data()->getJD( data()->UTime );

	bool isNewEpoch = false;
	if ( fabs( data()->CurrentDate - data()->CurrentEpoch ) > 365.25 ) { //update epoch-dependent numbers annually
		updateEpoch( data()->CurrentDate );
		isNewEpoch = true;
	}

  data()->LST = UTtoLST( data()->UTime, geo()->lng() );

	data()->LSTh.setH( data()->LST.hour(), data()->LST.minute(), data()->LST.second() );

  // Update positions of objects, if necessary
  // Sun and Planet positions change rather slowly, so only update them twice daily
  if ( fabs( data()->CurrentDate - data()->LastPlanetUpdate ) > 0.5 ) {
  	data()->LastPlanetUpdate = data()->CurrentDate;

		data()->Sun->findPosition( data()->CurrentDate );
		data()->Earth->findPosition( data()->CurrentDate );
		data()->Mercury->findPosition( data()->CurrentDate, data()->Earth );
		data()->Venus->findPosition( data()->CurrentDate, data()->Earth );
		data()->Mars->findPosition( data()->CurrentDate, data()->Earth );
		data()->Jupiter->findPosition( data()->CurrentDate, data()->Earth );
		data()->Saturn->findPosition( data()->CurrentDate, data()->Earth );
		data()->Uranus->findPosition( data()->CurrentDate, data()->Earth );
		data()->Neptune->findPosition( data()->CurrentDate, data()->Earth );
		data()->Pluto->findPosition( data()->CurrentDate, data()->Earth );

		//Recompute the Ecliptic
		data()->Ecliptic.clear();
		for ( unsigned int i=0; i<data()->Equator.count(); ++i ) {
			double ELong, ELat;
			ELong = data()->Equator.at(i)->ra().Degrees();
			ELat = 0.0;
			SkyPoint *o = new SkyPoint( 0.0, 0.0 );
			o->setEcliptic( ELong, ELat, data()->CurrentDate );
			data()->Ecliptic.append( o );
		}
	}

	// Moon moves ~30 arcmin/hr, so update its position every minute.
	if ( fabs( data()->CurrentDate - data()->LastMoonUpdate ) > 0.00069444 ) {
		data()->LastMoonUpdate = data()->CurrentDate;
		data()->Moon->findPosition( data()->CurrentDate, geo()->lat(), data()->LSTh );
		data()->Moon->findPhase( data()->Sun );
	}


	//update focus
	if ( options()->isTracking ) {
		//if tracking on a planet, update focus to keep up with changing position.
		bool onPlanet = false;
		if ( skymap->foundObject() == data()->Sun ) onPlanet = true;
		if ( skymap->foundObject() == data()->Moon ) onPlanet = true;
		if ( skymap->foundObject() == data()->Mercury ) onPlanet = true;
		if ( skymap->foundObject() == data()->Venus ) onPlanet = true;
		if ( skymap->foundObject() == data()->Mars ) onPlanet = true;
		if ( skymap->foundObject() == data()->Jupiter ) onPlanet = true;
		if ( skymap->foundObject() == data()->Saturn ) onPlanet = true;
		if ( skymap->foundObject() == data()->Uranus ) onPlanet = true;
		if ( skymap->foundObject() == data()->Neptune ) onPlanet = true;
		if ( skymap->foundObject() == data()->Pluto ) onPlanet = true;

		if ( onPlanet) skymap->setFocus( skymap->foundObject()->pos() );

		data()->HourAngle.setH( data()->LSTh.Hours() - skymap->focus()->ra().Hours() );
		skymap->focus()->EquatorialToHorizontal( data()->LSTh, geo()->lat() );
	} else {
		skymap->focus()->setRA( data()->LSTh.Hours() - data()->HourAngle.Hours() );
		skymap->focus()->EquatorialToHorizontal( data()->LSTh, geo()->lat() );
  }

	showFocusCoords();

	//Update Alt/Az coordinates.  Timescale varies with zoom level
	if ( fabs( data()->CurrentDate - data()->LastSkyUpdate) > 0.25/skymap->getPixelScale() ) {
		data()->LastSkyUpdate = data()->CurrentDate;

		//Recompute Alt, Az coords for all objects
		//Planets
		data()->Sun->pos()->EquatorialToHorizontal( data()->LSTh, geo()->lat() );
		data()->Moon->pos()->EquatorialToHorizontal( data()->LSTh, geo()->lat() );
		data()->Mercury->pos()->EquatorialToHorizontal( data()->LSTh, geo()->lat() );
		data()->Venus->pos()->EquatorialToHorizontal( data()->LSTh, geo()->lat() );
		data()->Mars->pos()->EquatorialToHorizontal( data()->LSTh, geo()->lat() );
		data()->Jupiter->pos()->EquatorialToHorizontal( data()->LSTh, geo()->lat() );
		data()->Saturn->pos()->EquatorialToHorizontal( data()->LSTh, geo()->lat() );
		data()->Uranus->pos()->EquatorialToHorizontal( data()->LSTh, geo()->lat() );
		data()->Neptune->pos()->EquatorialToHorizontal( data()->LSTh, geo()->lat() );
		data()->Pluto->pos()->EquatorialToHorizontal( data()->LSTh, geo()->lat() );

		//Stars
		if ( options()->drawBSC ) {
			for ( unsigned int i=0; i<data()->starList.count(); ++i ) {
				if (isNewEpoch) data()->starList.at(i)->pos()->updateCoords( data()->CurrentEpoch, data()->Obliquity, data()->dObliq, data()->dEcLong );
				data()->starList.at(i)->pos()->EquatorialToHorizontal( data()->LSTh, geo()->lat() );
			}
		}

		//Messier
		if ( options()->drawMessier || options()->drawMessImages ) {
			for ( unsigned int i=0; i<data()->messList.count(); ++i ) {
				if (isNewEpoch) data()->messList.at(i)->pos()->updateCoords( data()->CurrentEpoch, data()->Obliquity, data()->dObliq, data()->dEcLong );
				data()->messList.at(i)->pos()->EquatorialToHorizontal( data()->LSTh, geo()->lat() );
			}
		}

		//NGC
		if ( options()->drawNGC ) {
			for ( unsigned int i=0; i<data()->ngcList.count(); ++i ) {
				if (isNewEpoch) data()->ngcList.at(i)->pos()->updateCoords( data()->CurrentEpoch, data()->Obliquity, data()->dObliq, data()->dEcLong );
				data()->ngcList.at(i)->pos()->EquatorialToHorizontal( data()->LSTh, geo()->lat() );
			}
		}

		//IC
		if ( options()->drawIC ) {
			for ( unsigned int i=0; i<data()->icList.count(); ++i ) {
				if (isNewEpoch) data()->icList.at(i)->pos()->updateCoords( data()->CurrentEpoch, data()->Obliquity, data()->dObliq, data()->dEcLong );
				data()->icList.at(i)->pos()->EquatorialToHorizontal( data()->LSTh, geo()->lat() );
			}
		}

		//Milky Way
		if ( options()->drawMilkyWay ) {
			for ( unsigned int j=0; j<11; ++j ) {
				for ( unsigned int i=0; i<data()->MilkyWay[j].count(); ++i) {
					if (isNewEpoch) data()->MilkyWay[j].at(i)->updateCoords( data()->CurrentEpoch, data()->Obliquity, data()->dObliq, data()->dEcLong );
					data()->MilkyWay[j].at(i)->EquatorialToHorizontal( data()->LSTh, geo()->lat() );
				}
			}
		}

		//CLines
		if ( options()->drawConstellLines ) {
			for ( unsigned int i=0; i<data()->clineList.count(); ++i) {
				if (isNewEpoch) data()->clineList.at(i)->updateCoords( data()->CurrentEpoch, data()->Obliquity, data()->dObliq, data()->dEcLong );
				data()->clineList.at(i)->EquatorialToHorizontal( data()->LSTh, geo()->lat() );
			}
		}

		//CNames
		if ( options()->drawConstellNames ) {
			for ( unsigned int i=0; i<data()->cnameList.count(); ++i ) {
				if (isNewEpoch) data()->cnameList.at(i)->pos()->updateCoords( data()->CurrentEpoch, data()->Obliquity, data()->dObliq, data()->dEcLong );
				data()->cnameList.at(i)->pos()->EquatorialToHorizontal( data()->LSTh, geo()->lat() );
			}
		}

		//Celestial Equator
		if ( options()->drawEquator ) {
			for ( unsigned int i=0; i<data()->Equator.count(); ++i ) {
				data()->Equator.at(i)->EquatorialToHorizontal( data()->LSTh, geo()->lat() );
			}
		}

		//Ecliptic
		if ( options()->drawEcliptic ) {
			for ( unsigned int i=0; i<data()->Ecliptic.count(); ++i ) {
				data()->Ecliptic.at(i)->EquatorialToHorizontal( data()->LSTh, geo()->lat() );
			}
		}

		//Horizon: differnet than the others; Alt & Az remain constant, RA, Dec must keep up
		if ( options()->drawHorizon || options()->drawGround ) {
			for ( unsigned int i=0; i<data()->Horizon.count(); ++i ) {
				data()->Horizon.at(i)->HorizontalToEquatorial( data()->LSTh, geo()->lat() );
			}
		}

		skymap->setOldFocus( skymap->focus() );
		skymap->oldfocus()->EquatorialToHorizontal( data()->LSTh, geo()->lat() );
		skymap->Update();
	}
}

SkyObject* KStars::getObjectNamed( QString name ) {
	if ( (name== "star") || (name== "nothing") || name.isEmpty() ) return NULL;
	if ( name== data()->Sun->name() ) return data()->Sun;
	if ( name== data()->Moon->name() ) return data()->Moon;
	if ( name== data()->Mercury->name() ) return data()->Mercury;
	if ( name== data()->Venus->name() ) return data()->Venus;
	if ( name== data()->Mars->name() ) return data()->Mars;
	if ( name== data()->Jupiter->name() ) return data()->Jupiter;
	if ( name== data()->Saturn->name() ) return data()->Saturn;
	if ( name== data()->Uranus->name() ) return data()->Uranus;
	if ( name== data()->Neptune->name() ) return data()->Neptune;
	if ( name== data()->Pluto->name() ) return data()->Pluto;

	//Stars
	for ( unsigned int i=0; i<data()->starList.count(); ++i ) {
		if ( name==data()->starList.at(i)->name() ) return data()->starList.at(i);
	}

	//Messier
	for ( unsigned int i=0; i<data()->messList.count(); ++i ) {
		if ( name==data()->messList.at(i)->name() ) return data()->messList.at(i);
	}

	//NGC
	for ( unsigned int i=0; i<data()->ngcList.count(); ++i ) {
		if ( name==data()->ngcList.at(i)->name() ) return data()->ngcList.at(i);
	}

	//IC
	for ( unsigned int i=0; i<data()->icList.count(); ++i ) {
		if ( name==data()->icList.at(i)->name() ) return data()->icList.at(i);
	}

	//Constellations
	for ( unsigned int i=0; i<data()->cnameList.count(); ++i ) {
		if ( name==data()->cnameList.at(i)->name() ) return data()->cnameList.at(i);
	}

	//should never get here
	return NULL;
}

void KStars::showFocusCoords( void ) {
	//display object info in infoPanel
	QString s, oname;
	char dsgn = '+', azsgn = '+', altsgn = '+';

	oname = tr( "nothing" );
	if ( skymap->foundObject() != NULL && options()->isTracking ) {
		oname = skymap->foundObject()->translatedName();
		//add genetive name for stars
	  if ( skymap->foundObject()->type()==0 && skymap->foundObject()->name2().length() )
			oname += " (" + skymap->foundObject()->name2() + ")";
	}

	if ( skymap->focus()->dec().Degrees() < 0 ) dsgn = '-';
	int dd = abs( skymap->focus()->dec().degree() );
	int dm = abs( skymap->focus()->dec().getArcMin() );
	int ds = abs( skymap->focus()->dec().getArcSec() );
	FocusRA->setText( tr( "RA", "Right Ascension" ) + s.sprintf( ": %02d:%02d:%02d", skymap->focus()->ra().hour(), skymap->focus()->ra().minute(),  skymap->focus()->ra().second() ) );
	FocusDec->setText( tr( "Dec", "Declination" ) +  s.sprintf( ": %c%02d:%02d:%02d", dsgn, dd, dm, ds ) );

	if ( skymap->focus()->alt().Degrees() < 0 ) altsgn = '-';
	if ( skymap->focus()->az().Degrees() < 0 ) azsgn = '-';
	int altd = abs( skymap->focus()->alt().degree() );
	int altm = abs( skymap->focus()->alt().getArcMin() );
	int alts = abs( skymap->focus()->alt().getArcSec() );
	int azd = abs( skymap->focus()->az().degree() );
	int azm = abs( skymap->focus()->az().getArcMin() );
	int azs = abs( skymap->focus()->az().getArcSec() );
	FocusAlt->setText( tr( "Alt", "Altitude" ) + s.sprintf( ": %c%02d:%02d:%02d", altsgn, altd, altm, alts ) );
	FocusAz->setText( tr( "Az", "Azimuth" ) + s.sprintf( ": %c%02d:%02d:%02d", azsgn, azd, azm, azs ) );
}

QString KStars::getDateString( QDate date ) {
  QString dummy;
  QString dateString = dummy.sprintf( "%02d / %02d / %04d",
			  date.month(), date.day(), date.year() );
  return dateString;
}

/*
long double KStars::getJD( QDateTime t ) {
  int year = t.date().year();
  int month = t.date().month();
  int day = t.date().day();
  int hour = t.time().hour();
  int minute = t.time().minute();
  double second = t.time().second() + 0.001*t.time().msec();
  int m, y, A, B, C, D;

  if (month > 2) {
	  m = month;
	  y = year;
  } else {
	  y = year - 1;
	  m = month + 12;
  }

//  If the date is after 10/15/1582, then take Pope Gregory's modification
//	 to the Julian calendar into account

	 if (( year  >1582 ) ||
		 ( year ==1582 && month  >9 ) ||
		 ( year ==1582 && month ==9 && day >15 ))
	 {
		 A = int(y/100);
		 B = 2 - A + int(A/4);
	 } else {
		 B = 0;
	 }

  if (y < 0) {
	  C = int((365.25*y) - 0.75);
  } else {
	  C = int(365.25*y);
  }

  D = int(30.6001*(m+1));

  long double d = double(day) + (double(hour) + (double(minute) + second/60.0)/60.0)/24.0;
  long double jd = B + C + D + d + 1720994.5;

  return jd;
}
*/

void KStars::updateEpoch( long double NewEpoch ) {
	//Several quantities change slowly with time:
	//Obliquity, Ecliptic Longitude,
	//eccentricity of Earth's Orbit
	//Earth's longitude of perihelion
	data()->CurrentEpoch = NewEpoch;
	data()->Obliquity = findObliquity( data()->CurrentEpoch );
	nutate( data()->CurrentEpoch ); //find dEcLong and dObliq
//	Obliquity += dObliq/3600.0;
}

double KStars::findObliquity( long double JD ) {
	double U = ( JD - J2000 )/3652500.0;
	double dObliq = -4680.93*U - 1.55*U*U + 1999.25*U*U*U
									- 51.38*U*U*U*U - 249.67*U*U*U*U*U
									- 39.05*U*U*U*U*U*U + 7.12*U*U*U*U*U*U*U
									+ 27.87*U*U*U*U*U*U*U*U + 5.79*U*U*U*U*U*U*U*U*U
									+ 2.45*U*U*U*U*U*U*U*U*U*U;
	double Obliq = 23.43929111 + dObliq/3600.0;
	return Obliq;
}

void KStars::nutate( long double JD ) {
	//This uses the pre-1984 theory of nutation.  The results
	//are accurate to 0.5 arcsec, should be fine for KStars.
	//calculations from "Practical Astronomy with Your Calculator"
	//by Peter Duffett-Smith
	//
	//Nutation is a small, quasi-periodic oscillation of the Earth's
	//spin axis (in addition to the more regular precession).  It
	//is manifested as a change in the zeropoint of the Ecliptic
	//Longitude (dEcLong) and a change in the Obliquity angle (dObliq)
	//see the Astronomy Help pages for more info.

	dms  L2, M2, O, O2;
	double T;
	double sin2L, cos2L, sin2M, cos2M;
	double sinO, cosO, sin2O, cos2O;

	T = ( JD - J2000 )/36525.0; //centuries since J2000.0
  O.setD( 125.04452 - 1934.136261*T + 0.0020708*T*T + T*T*T/450000.0 ); //ecl. long. of Moon's ascending node
	O2.setD( 2.0*O.Degrees() );
	L2.setD( 2.0*( 280.4665 + 36000.7698*T ) ); //twice mean ecl. long. of Sun
	M2.setD( 2.0*( 218.3165 + 481267.8813*T ) );//twice mean ecl. long. of Moon

	O.SinCos( sinO, cosO );
	O2.SinCos( sin2O, cos2O );
	L2.SinCos( sin2L, cos2L );
	M2.SinCos( sin2M, cos2M );

	data()->dEcLong = ( -17.2*sinO - 1.32*sin2L - 0.23*sin2M + 0.21*sin2O)/3600.0; //Ecl. long. correction
	data()->dObliq = (   9.2*cosO + 0.57*cos2L + 0.10*cos2M - 0.09*cos2O)/3600.0; //Obliq. correction
}

QTime KStars::UTtoLST( QDateTime UT, dms longitude) {
  QTime GST = UTtoGST( UT );
  QTime LST = GSTtoLST( GST, longitude );
  return LST;
}

QTime KStars::UTtoGST( QDateTime UT ) {
  QDateTime t0 = UT;
  t0.setTime( QTime(0,0,0) );
  long double jd0 = data()->getJD( t0 );
  long double s = jd0 - 2451545.0;
  double t = s/36525.0;
  double t1 = 6.697374558 + 2400.051336*t + 0.000025862*t*t;

	 while (t1 >= 24.0) {t1 -= 24.0;}
	 while (t1 <   0.0) {t1 += 24.0;}

  double hr = double( UT.time().hour() );
  double mn = double( UT.time().minute() );
  double sc = double( UT.time().second() ) + double ( 0.001 * UT.time().msec() );

  double t2 = (hr + ( mn + sc/60.0)/60.0)*1.002737909;
  double gsth = t1 + t2;

  while (gsth < 0.0) gsth += 24.0;
  while (gsth >24.0) gsth -= 24.0;

  int gh = int(gsth);
  int gm = int((gsth - gh)*60.0);
  int gs = int(((gsth - gh)*60.0 - gm)*60.0);

  QTime gst(gh, gm, gs);

  return gst;
}

QTime KStars::GSTtoLST( QTime GST, dms longitude) {
  double lsth = double(GST.hour()) + (double(GST.minute()) +
			   double(GST.second())/60.0)/60.0 + longitude.Degrees()/15.0;

  while (lsth < 0.0) lsth += 24.0;
  while (lsth >24.0) lsth -= 24.0;

  int lh = int(lsth);
  int lm = int((lsth - lh)*60.0);
  int ls = int(((lsth - lh)*60.0 - lm)*60.0);

  QTime lst(lh, lm, ls);

  return lst;
}

void KStars::closeEvent (QCloseEvent *e)
{
	if (!skymap->runningDownloads())
	{
//		emit closeAllWindows();
		e->accept();
		qApp->quit();   // quits the program
	}
	else
		e->ignore();
}

void KStars::changeTimeStep( void ) {
	data()->SysJD_Mark = data()->getJD( QDateTime::currentDateTime() );
	data()->SkyJD_Mark = data()->getJD( data()->LTime );
}

void KStars::setAltAz(double alt, double az) {
 	skymap->setFocusAltAz(alt,az);
}

void KStars::lookTowards (QString direction) {
 	if (direction == "zenith")   mZenith();
 	else if (direction == "north")     skymap->setFocusAltAz(15.0, 0.0);
 	else if (direction == "northeast") skymap->setFocusAltAz(15.0, 45.0);
 	else if (direction == "east")      skymap->setFocusAltAz(15.0, 90.0);
 	else if (direction == "southeast") skymap->setFocusAltAz(15.0, 135.0);
	else if (direction == "south")     skymap->setFocusAltAz(15.0, 180.0);
	else if (direction == "southwest") skymap->setFocusAltAz(15.0, 225.0);
 	else if (direction == "west")      skymap->setFocusAltAz(15.0, 270.0);
 	else if (direction == "northwest") skymap->setFocusAltAz(15.0, 315.0);
}

void KStars::setLocalTime(int yr, int mth, int day, int hr, int min, int sec) {

 	changeTime( QDate(yr, mth, day), QTime(hr,min,sec));
}

void KStars::controlTimer(QString state) {
	if (state == "stop") {
		if ( tmr->isActive() ) {
			tmr->stop();
			actTimeRun->setIconSet( Resource::loadIconSet( "kstars/1rightarrow" ) );
			actTimeRun->setText( tr( "Start Clock" ) );
			actTimeRun->setToolTip( tr( "Start Clock" ) );
		}
	} else if (state == "start") {
		if (! tmr->isActive() ) {
			data()->SysJD_Mark = data()->getJD( QDateTime::currentDateTime() );
			data()->SkyJD_Mark = data()->getJD( data()->LTime );

			tmr->start( TIMER_INTERVAL, FALSE );
			actTimeRun->setIconSet( Resource::loadIconSet( "kstars/player_pause" ) );
			actTimeRun->setText( tr( "Stop Clock" ) );
			actTimeRun->setToolTip( tr( "Stop Clock" ) );
		}
	}
}

