/***************************************************************************
                          viewopsdialog.cpp  -  K Desktop Planetarium
                             -------------------
    begin                : Sat Apr 28 2001
    copyright            : (C) 2001 by Jason Harris
    email                : jharris@30doradus.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <qspinbox.h>
#include "lineeditdlg.h"
#include <qmessagebox.h>

#include <qtextstream.h>
#include <qlayout.h>
#include <qtabwidget.h>
#include <qwidget.h>
#include <qgroupbox.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qcheckbox.h>
#include <qframe.h>
#include <qbutton.h>
#include <qcolor.h>
#include <qfont.h>
#include "colordialog.h"
#include <qlabel.h>
#include <qlistbox.h>
#include <qpushbutton.h>
#include <qcombobox.h>
#include <qpe/global.h>

#include "kstars.h"
#include "magnitudespinbox.h"
#include "viewopsdialog.h"

ViewOpsDialog::ViewOpsDialog( QWidget *parent )
	: QDialog( parent, tr( "Display Options" ), TRUE ) {
	ksw = (KStars *)parent;

	// Layout manager for entire dialog
	vlay = new QVBoxLayout( this, 2, 2 );

	DisplayTabs = new QTabWidget( this, "DisplayTabs" );

	vlay->addWidget( DisplayTabs );

	CoordsTab = new QWidget( DisplayTabs, "CoordsTab" );
	vlayCoordsTab = new QVBoxLayout( CoordsTab, 2 );

	CoordsGroup = new QButtonGroup( tr( "Coordinate system" ), CoordsTab );
	CoordsGroup->setColumnLayout( 0, Qt::Vertical );
	CoordsGroup->layout()->setSpacing( 0 );
	CoordsGroup->layout()->setMargin( 0 );

	CoordsGroupLayout = new QVBoxLayout( CoordsGroup->layout(), 2 );
	CoordsGroupLayout->setAlignment( Qt::AlignTop );
	CoordsGroupLayout->setMargin( 4 );

	EquatRadio = new QRadioButton( tr( "Equatorial" ), CoordsGroup );
	EquatRadio->setChecked( !ksw->options()->useAltAz );

	AltAzRadio = new QRadioButton( tr( "Horizontal" ), CoordsGroup );
	AltAzRadio->setChecked( ksw->options()->useAltAz );

	CoordsGroupLayout->addWidget( EquatRadio );
	CoordsGroupLayout->addWidget( AltAzRadio );

	vlayCoordsTab->addWidget( CoordsGroup );
	vlayCoordsTab->addStretch();

	DisplayTabs->insertTab( CoordsTab, tr( "General" ) );

	// construct stars tab
	StarsTab = new  QWidget( DisplayTabs, "StarsTab" );
	vlayStarsTab = new QVBoxLayout( StarsTab, 2 );

	showBSC = new QCheckBox( tr( "SAO Star Catalog" ), StarsTab );
	QFont stdFont(  showBSC->font() );

	showBSC->setFont( stdFont );
	showBSC->setChecked( ksw->options()->drawBSC );

	// Spin box : show stars brighter than magnitude limit
	int intMagLimitDrawStars = int ( 10.0 * ksw->options()->magLimitDrawStar );

// faintLimit doesn't work with reloading star data
//	int faintLimit = int( 10.*ksw->data()->starList.at(ksw->data()->starList.count()-1)->mag );
//	magSpinBoxDrawStars = new MagnitudeSpinBox( 0, faintLimit, StarsTab );
	magSpinBoxDrawStars = new MagnitudeSpinBox( 0, 79, StarsTab );	// max mag = 7.9
	magSpinBoxDrawStars->setFont( stdFont );
	magSpinBoxDrawStars->setValue( intMagLimitDrawStars );

	QLabel *textLabelMagStars = new QLabel( StarsTab, "LabelMagStars" );
	textLabelMagStars->setText( tr( "Show stars brighter than" ) );
	textLabelMagStars->setFont( stdFont );

	// Spin box : show stars names for stars brighter than magnitude limit
	int intMagLimitDrawStarInfo = 10*((int)(ksw->options()->magLimitDrawStarInfo));
	//	float magLimitDrawStarName;
	magSpinBoxDrawStarInfo = new MagnitudeSpinBox( 0, 90, StarsTab );
	magSpinBoxDrawStarInfo->setFont( stdFont );
	magSpinBoxDrawStarInfo->setValue( intMagLimitDrawStarInfo );

	QLabel *textLabelMagStarInfo = new QLabel( StarsTab, "LabelMagStarNames" );
	textLabelMagStarInfo->setText( tr( "for stars brighter than" ) );
	textLabelMagStarInfo->setFont( stdFont );

	showStarNames = new QCheckBox( tr( "show name" ), StarsTab );
	showStarNames->setFont( stdFont );
	showStarNames->setChecked( ksw->options()->drawStarName );

	showStarMagnitude = new QCheckBox( tr( "show magnitude" ), StarsTab );
	showStarMagnitude->setFont( stdFont );
	showStarMagnitude->setChecked( ksw->options()->drawStarMagnitude );

	// MagnitudeSpinBox* magSpinBoxDrawStarNames;


	vlayStarsTab->addWidget( showBSC );
	vlayStarsTab->addWidget( textLabelMagStars );
	vlayStarsTab->addWidget( magSpinBoxDrawStars );
	vlayStarsTab->addSpacing( 6 );
	vlayStarsTab->addWidget( textLabelMagStarInfo );
	vlayStarsTab->addWidget( magSpinBoxDrawStarInfo );
	vlayStarsTab->addWidget( showStarNames );
	vlayStarsTab->addWidget( showStarMagnitude );
	vlayStarsTab->addStretch();

	DisplayTabs->insertTab( StarsTab, tr( "Stars" ) );

//Construct Catalogs tab:
	CatalogTab = new QWidget( DisplayTabs, "CatalogTab" );
	vlayCatTab = new QVBoxLayout( CatalogTab, 2 );

	showMessier = new QCheckBox( tr( "Messier Objects (symbols)" ), CatalogTab );
	showMessier->setFont( stdFont );
	showMessier->setChecked( ksw->options()->drawMessier );

	showMessImages = new QCheckBox( tr( "Messier Objects (images)" ), CatalogTab );
	showMessImages->setFont( stdFont );
	showMessImages->setChecked( ksw->options()->drawMessImages );

	showNGC = new QCheckBox( tr( "NGC Catalog" ), CatalogTab );
	showNGC->setFont( stdFont );
	showNGC->setChecked( ksw->options()->drawNGC );

	showIC = new QCheckBox( tr( "IC Catalog" ), CatalogTab );
	showIC->setFont( stdFont );
	showIC->setChecked( ksw->options()->drawIC );

	vlayCatTab->addWidget( showMessier );
	vlayCatTab->addWidget( showMessImages );
	vlayCatTab->addWidget( showNGC );
	vlayCatTab->addWidget( showIC );
	vlayCatTab->addStretch();

	DisplayTabs->insertTab( CatalogTab, tr( "Catalogs" ) );

//Construct Guides tab:
	GuideTab = new QWidget( DisplayTabs, "GuideTab" );
	vlayGuideTab = new QVBoxLayout( GuideTab, 2 );
	glayGuideTab = new QGridLayout( 5, 2, 2 );

	showConstellLines = new QCheckBox( tr( "Constellation Lines" ), GuideTab );
	showConstellLines->setFont( stdFont );
	showConstellLines->setChecked( ksw->options()->drawConstellLines );

	showConstellNames = new QCheckBox( tr( "Constellation Names" ), GuideTab );
	showConstellNames->setFont( stdFont );
	showConstellNames->setChecked( ksw->options()->drawConstellNames );
	ConstellOptions = new QButtonGroup( tr( "Constellation Name Options" ), GuideTab );
	ConstellOptions->setColumnLayout( 0, Qt::Vertical );
	ConstellOptions->layout()->setSpacing( 0 );
	ConstellOptions->layout()->setMargin( 0 );
	ConstellOptionsLayout = new QVBoxLayout( ConstellOptions->layout(), 2 );
	ConstellOptionsLayout->setAlignment( Qt::AlignTop );
	ConstellOptionsLayout->setMargin( 4 );
	useLatinConstellNames = new QRadioButton( tr( "Latin" ), ConstellOptions );
	useLatinConstellNames->setFont ( stdFont );
	useLatinConstellNames->setChecked( ksw->options()->useLatinConstellNames );
  useLocalConstellNames = new QRadioButton( tr( "Localized" ), ConstellOptions );
	useLocalConstellNames->setFont ( stdFont );
	useLocalConstellNames->setChecked( ksw->options()->useLocalConstellNames );
  useAbbrevConstellNames = new QRadioButton( tr( "Abbreviated" ), ConstellOptions );
	useAbbrevConstellNames->setFont ( stdFont );
	useAbbrevConstellNames->setChecked( ksw->options()->useAbbrevConstellNames );
	ConstellOptions->setEnabled( showConstellNames->isChecked() );
	ConstellOptionsLayout->addWidget( useLatinConstellNames );
	ConstellOptionsLayout->addWidget( useLocalConstellNames );
	ConstellOptionsLayout->addWidget( useAbbrevConstellNames );

	showMilkyWay = new QCheckBox( tr( "Milky Way" ), GuideTab );
	showMilkyWay->setFont( stdFont );
	showMilkyWay->setChecked( ksw->options()->drawMilkyWay );

	showMilkyWayFilled = new QCheckBox( tr( "Fill Milky Way" ), GuideTab );
	showMilkyWayFilled->setFont( stdFont );
	showMilkyWayFilled->setChecked( ksw->options()->fillMilkyWay );
  showMilkyWayFilled->setEnabled( showMilkyWay->isChecked() );

	showGrid = new QCheckBox( tr( "Coordinate Grid" ), GuideTab );
	showGrid->setFont( stdFont );
	showGrid->setChecked( ksw->options()->drawGrid );

	showEquator = new QCheckBox( tr( "Celestial Equator" ), GuideTab );
	showEquator->setFont( stdFont );
	showEquator->setChecked( ksw->options()->drawEquator );

	showEcliptic = new QCheckBox( tr( "Ecliptic" ), GuideTab );
	showEcliptic->setFont( stdFont );
	showEcliptic->setChecked( ksw->options()->drawEcliptic );

	showHorizon = new QCheckBox( tr( "Horizon (line)" ), GuideTab );
	showHorizon->setFont( stdFont );
	showHorizon->setChecked( ksw->options()->drawHorizon );

	showGround = new QCheckBox( tr( "Opaque Ground" ), GuideTab );
	showGround->setFont( stdFont );
	showGround->setChecked( ksw->options()->drawGround );
	if ( !ksw->options()->useAltAz ) showGround->setEnabled( false );

	vlayGuideTab->addWidget( showConstellLines );
	vlayGuideTab->addWidget( showConstellNames );
	vlayGuideTab->addWidget( ConstellOptions );
	vlayGuideTab->addSpacing( 6 );
	glayGuideTab->addWidget( showMilkyWay, 0, 0 );
	glayGuideTab->addWidget( showMilkyWayFilled, 1, 0 );
	glayGuideTab->addWidget( showGrid, 0, 1 );
	glayGuideTab->addWidget( showEquator, 1, 1 );
	glayGuideTab->addWidget( showEcliptic, 2, 1 );
	glayGuideTab->addWidget( showHorizon, 3, 1 );
	glayGuideTab->addWidget( showGround, 4, 1 );
	vlayGuideTab->addLayout( glayGuideTab );
	vlayGuideTab->addStretch();

	DisplayTabs->insertTab( GuideTab, tr( "Guides" ) );

//Construct Planets tab:
	PlanetTab = new QWidget( DisplayTabs, "PlanetTab" );
	vlayPlanetTab = new QVBoxLayout( PlanetTab, 2 );
	glayPlanetTab = new QGridLayout( 7, 2, 2 );

	showSun = new QCheckBox( tr( "The Sun" ), PlanetTab );
	showSun->setFont( stdFont );
	showSun->setChecked( ksw->options()->drawSun );

	showMoon = new QCheckBox( tr( "The Moon" ), PlanetTab );
  showMoon->setFont( stdFont );
  showMoon->setChecked( ksw->options()->drawMoon );
  showMoon->setChecked( ksw->options()->drawMoon );

	showMercury = new QCheckBox( tr( "Mercury" ), PlanetTab );
	showMercury->setFont( stdFont );
	showMercury->setChecked( ksw->options()->drawMercury );

	showVenus = new QCheckBox( tr( "Venus" ), PlanetTab );
	showVenus->setFont( stdFont );
	showVenus->setChecked( ksw->options()->drawVenus );

	showMars = new QCheckBox( tr( "Mars" ), PlanetTab );
	showMars->setFont( stdFont );
	showMars->setChecked( ksw->options()->drawMars );

	showJupiter = new QCheckBox( tr( "Jupiter" ), PlanetTab );
	showJupiter->setFont( stdFont );
	showJupiter->setChecked( ksw->options()->drawJupiter );

	showSaturn = new QCheckBox( tr( "Saturn" ), PlanetTab );
	showSaturn->setFont( stdFont );
	showSaturn->setChecked( ksw->options()->drawSaturn );

	showUranus = new QCheckBox( tr( "Uranus" ), PlanetTab );
	showUranus->setFont( stdFont );
	showUranus->setChecked( ksw->options()->drawUranus );

	showNeptune = new QCheckBox( tr( "Neptune" ), PlanetTab );
	showNeptune->setFont( stdFont );
	showNeptune->setChecked( ksw->options()->drawNeptune );

	showPluto = new QCheckBox( tr( "Pluto" ), PlanetTab );
	showPluto->setFont( stdFont );
	showPluto->setChecked( ksw->options()->drawPluto );

	showAll = new QPushButton( tr( "Show All" ), PlanetTab );
	showAll->setFont( stdFont );

	showNone = new QPushButton( tr( "Show None" ), PlanetTab );
	showNone->setFont( stdFont );

	glayPlanetTab->addWidget( showSun, 0, 0 );
	glayPlanetTab->addWidget( showMoon, 1, 0 );
	glayPlanetTab->addWidget( showMercury, 2, 0 );
	glayPlanetTab->addWidget( showVenus, 3, 0 );
	glayPlanetTab->addWidget( showMars, 4, 0 );
	glayPlanetTab->addWidget( showJupiter, 0, 1 );
	glayPlanetTab->addWidget( showSaturn, 1, 1 );
	glayPlanetTab->addWidget( showUranus, 2, 1 );
	glayPlanetTab->addWidget( showNeptune, 3, 1 );
	glayPlanetTab->addWidget( showPluto, 4, 1 );
	glayPlanetTab->addWidget( showAll, 6, 0 );
	glayPlanetTab->addWidget( showNone, 6, 1 );

	vlayPlanetTab->addLayout( glayPlanetTab );
  vlayPlanetTab->addStretch();

	DisplayTabs->insertTab( PlanetTab, tr( "Solar System" ) );

//Construct the Color Tab
	ColorTab = new QWidget( DisplayTabs, "ColorTab" );
	TopBox = new QGroupBox( ColorTab, "TopBox" );
	TopBox->setColumnLayout( 0, Qt::Vertical );
	TopBox->setTitle( tr( "Current Colors" ) );
	TopBox->layout()->setSpacing( 0 );
	TopBox->layout()->setMargin( 0 );

	BottomBox = new QGroupBox( ColorTab, "BottomBox" );
	BottomBox->setColumnLayout( 0, Qt::Vertical );
	BottomBox->setTitle( tr( "Preset Color Schemes" ) );
	BottomBox->layout()->setSpacing( 0 );
	BottomBox->layout()->setMargin( 0 );

  vlayColorTab = new QVBoxLayout( ColorTab );
	vlayTopBox = new QVBoxLayout( TopBox->layout(), 2 );
	vlayTopBox->setAlignment( Qt::AlignTop );
  hlayTopBox = new QHBoxLayout( 2 );
  hlayTopBox2 = new QHBoxLayout( 2 );
	vlayBottomBox = new QVBoxLayout( BottomBox->layout(), 2 );
	vlayBottomBox->setAlignment( Qt::AlignTop );

	vlayColorTab->setSpacing( 2 );
	vlayColorTab->setMargin( 2 );
	vlayTopBox->setMargin( 4 );
	vlayBottomBox->setMargin( 4 );

	ColorPalette = new QListBox( TopBox );
  ColorPalette->setSelectionMode( QListBox::Single );

	SkyColor = new QPixmap( 18, 12 );
	SkyColor->fill( QColor( ksw->options()->colorSky ) );
	ColorPalette->insertItem( *SkyColor, tr( "Sky" ) );

	MessColor = new QPixmap( 18, 12 );
	MessColor->fill( QColor( ksw->options()->colorMess ) );
	ColorPalette->insertItem( *MessColor, tr( "Messier Object" ) );

	NGCColor = new QPixmap( 18, 12 );
	NGCColor->fill( QColor( ksw->options()->colorNGC ) );
	ColorPalette->insertItem( *NGCColor, tr( "NGC Object" ) );

	ICColor = new QPixmap( 18, 12 );
	ICColor->fill( QColor( ksw->options()->colorIC ) );
	ColorPalette->insertItem( *ICColor, tr( "IC Object" ) );

	LinksColor = new QPixmap( 18, 12 );
	LinksColor->fill( QColor( ksw->options()->colorHST ) );
	ColorPalette->insertItem( *LinksColor, tr( "Object w/ Links" ) );

	SNameColor = new QPixmap( 18, 12 );
	SNameColor->fill( QColor( ksw->options()->colorSName ) );
	ColorPalette->insertItem( *SNameColor, tr( "Star Name" ) );

	CNameColor = new QPixmap( 18, 12 );
	CNameColor->fill( QColor( ksw->options()->colorCName ) );
	ColorPalette->insertItem( *CNameColor, tr( "Constell. Name" ) );

	CLineColor = new QPixmap( 18, 12 );
	CLineColor->fill( QColor( ksw->options()->colorCLine ) );
	ColorPalette->insertItem( *CLineColor, tr( "Constell. Line" ) );

	MWColor = new QPixmap( 18, 12 );
	MWColor->fill( QColor( ksw->options()->colorMW ) );
	ColorPalette->insertItem( *MWColor, tr( "Milky Way" ) );

	EqColor = new QPixmap( 18, 12 );
	EqColor->fill( QColor( ksw->options()->colorEq ) );
	ColorPalette->insertItem( *EqColor, tr( "Equator" ) );

	EclColor = new QPixmap( 18, 12 );
	EclColor->fill( QColor( ksw->options()->colorEcl ) );
	ColorPalette->insertItem( *EclColor, tr( "Ecliptic" ) );

	HorzColor = new QPixmap( 18, 12 );
	HorzColor->fill( QColor( ksw->options()->colorHorz ) );
	ColorPalette->insertItem( *HorzColor, tr( "Horizon" ) );

	GridColor = new QPixmap( 18, 12 );
	GridColor->fill( QColor( ksw->options()->colorGrid ) );
	ColorPalette->insertItem( *GridColor, tr( "Coordinate Grid" ) );

// the spinbox for colorintensity of stars
	IntensityBox = new QSpinBox (0, 10, 1, TopBox);
	IntensityBox->setValue( ksw->options()->starColorIntensity );
	QLabel *IntensityLabel = new QLabel (IntensityBox, tr ("Star Color Intensity"), TopBox);
	IntensityLabel->setAlignment ( AlignRight | AlignVCenter );

// the QComboBox for the starcolor mode
	StarColorMode = new QComboBox( TopBox );
	QLabel *ColorModeLabel = new QLabel( StarColorMode, tr( "Star Color Mode" ), TopBox );
	ColorModeLabel->setAlignment ( AlignRight | AlignVCenter );
	StarColorMode->insertItem( tr( "Real Colors" ) );
	StarColorMode->insertItem( tr( "Solid Red" ) );
	StarColorMode->insertItem( tr( "Solid Black" ) );
	StarColorMode->insertItem( tr( "Solid White" ) );

//The list of preset color schemes
	PresetBox = new QListBox( BottomBox );
  PresetBox->setSelectionMode( QListBox::Single );
	PresetBox->insertItem( tr( "Default Colors" ) );
	PresetBox->insertItem( tr( "Star Chart" ) );
	PresetBox->insertItem( tr( "Night Vision" ) );
	PresetBox->setMinimumHeight( PresetBox->sizeHint().height() + 4 );

	PresetFileList.append( "default.colors" );
	PresetFileList.append( "chart.colors" );
	PresetFileList.append( "night.colors" );

	QFile file;
	QString line, schemeName, filename;
	file.setName( Global::applicationFileName( "kstars", "colors.dat" ) ); //determine filename in local user KDE directory tree.
	if ( file.open( IO_ReadOnly ) ) {
		QTextStream stream( &file );

  	while ( !stream.eof() ) {
			line = stream.readLine();
			schemeName = line.left( line.find( ':' ) );
			filename = line.mid( line.find( ':' ) +1, line.length() );
			PresetBox->insertItem( schemeName );
			PresetFileList.append( filename );
	  }
		file.close();
	}

	AddPreset = new QPushButton( tr( "Save Current Colors" ), BottomBox );

//Pack the widgets into the layouts
	hlayTopBox2->addWidget ( IntensityLabel );
	hlayTopBox2->addWidget ( IntensityBox );

	hlayTopBox->addWidget ( ColorModeLabel );
	hlayTopBox->addWidget ( StarColorMode );

	vlayTopBox->addWidget( ColorPalette );
	vlayTopBox->addLayout( hlayTopBox );
	vlayTopBox->addLayout( hlayTopBox2 );

	vlayBottomBox->addWidget( PresetBox );
	vlayBottomBox->addWidget( AddPreset );

	vlayColorTab->addWidget ( TopBox );
	vlayColorTab->addWidget ( BottomBox );

	DisplayTabs->insertTab( ColorTab, tr( "Colors" ) );

	vlay->activate();

	connect( ColorPalette, SIGNAL( clicked( QListBoxItem* ) ), this, SLOT( newColor( QListBoxItem* ) ) );
	connect( PresetBox, SIGNAL( highlighted( int ) ), this, SLOT( slotPreset( int ) ) );
	connect( AddPreset, SIGNAL( clicked() ), this, SLOT( slotAddPreset() ) );
	connect( showBSC, SIGNAL( clicked() ), this, SLOT( updateDisplay() ) );
	connect( showMessier, SIGNAL( clicked() ), this, SLOT( updateDisplay() ) );
	connect( showMessImages, SIGNAL( clicked() ), this, SLOT( updateDisplay() ) );
	connect( showNGC, SIGNAL( clicked() ), this, SLOT( updateDisplay() ) );
	connect( showIC, SIGNAL( clicked() ), this, SLOT( updateDisplay() ) );
	connect( showConstellLines, SIGNAL( clicked() ), this, SLOT( updateDisplay() ) );
	connect( showConstellNames, SIGNAL( clicked() ), this, SLOT( updateDisplay() ) );
	connect( useLatinConstellNames, SIGNAL( clicked() ), this, SLOT( updateDisplay() ) );
	connect( useLocalConstellNames, SIGNAL( clicked() ), this, SLOT( updateDisplay() ) );
	connect( useAbbrevConstellNames, SIGNAL( clicked() ), this, SLOT( updateDisplay() ) );
	connect( showMilkyWay, SIGNAL( clicked() ), this, SLOT( updateDisplay() ) );
	connect( showMilkyWayFilled, SIGNAL( clicked() ), this, SLOT( updateDisplay() ) );
	connect( showGrid, SIGNAL( clicked() ), this, SLOT( updateDisplay() ) );
	connect( showEquator, SIGNAL( clicked() ), this, SLOT( updateDisplay() ) );
	connect( showEcliptic, SIGNAL( clicked() ), this, SLOT( updateDisplay() ) );
	connect( showHorizon, SIGNAL( clicked() ), this, SLOT( updateDisplay() ) );
	connect( showGround, SIGNAL( clicked() ), this, SLOT( updateDisplay() ) );
	connect( showSun, SIGNAL( clicked() ), this, SLOT( updateDisplay() ) );
	connect( showMoon, SIGNAL( clicked() ), this, SLOT( updateDisplay() ) );
	connect( showMercury, SIGNAL( clicked() ), this, SLOT( updateDisplay() ) );
	connect( showVenus, SIGNAL( clicked() ), this, SLOT( updateDisplay() ) );
	connect( showMars, SIGNAL( clicked() ), this, SLOT( updateDisplay() ) );
	connect( showJupiter, SIGNAL( clicked() ), this, SLOT( updateDisplay() ) );
	connect( showSaturn, SIGNAL( clicked() ), this, SLOT( updateDisplay() ) );
	connect( showUranus, SIGNAL( clicked() ), this, SLOT( updateDisplay() ) );
	connect( showNeptune, SIGNAL( clicked() ), this, SLOT( updateDisplay() ) );
	connect( showPluto, SIGNAL( clicked() ), this, SLOT( updateDisplay() ) );
	connect( showAll, SIGNAL( clicked() ), this, SLOT( markPlanets() ) );
	connect( showNone, SIGNAL( clicked() ), this, SLOT( unMarkPlanets() ) );
	connect( EquatRadio, SIGNAL( clicked() ), this, SLOT( changeCoordSys() ) );
	connect( AltAzRadio, SIGNAL( clicked() ), this, SLOT( changeCoordSys() ) );
	connect( magSpinBoxDrawStars, SIGNAL( valueChanged( int ) ), SLOT( changeMagDrawStars( int) ) );
	connect( magSpinBoxDrawStarInfo, SIGNAL( valueChanged( int ) ), SLOT( changeMagDrawInfo( int) ) );
	connect( showStarNames, 		SIGNAL( clicked() ), this, SLOT( updateDisplay() ) );
	connect( showStarMagnitude, SIGNAL( clicked() ), this, SLOT( updateDisplay() ) );
	connect( IntensityBox, SIGNAL (valueChanged (int)), SLOT (changeStarColorIntensity(int)) );
	connect( StarColorMode, SIGNAL( activated( int ) ), this, SLOT( changeStarColorMode( int ) ) );
}

ViewOpsDialog::~ViewOpsDialog(){
	delete SkyColor;  delete MessColor;  delete NGCColor;  delete ICColor;
	delete LinksColor;  delete SNameColor;  delete CLineColor;  delete CNameColor;
	delete EqColor;  delete EclColor;  delete HorzColor;  delete GridColor;
	delete MWColor;
}

void ViewOpsDialog::changeMagDrawStars( int newValue )
{
	float fNewValue = ( newValue * 1.0) / 10.0;

	ksw->data()->setMagnitude( fNewValue );

	// force redraw
	ksw->skymap->Update();
}

void ViewOpsDialog::changeMagDrawInfo( int newValue )
{
	float fNewValue = ( newValue * 1.0) / 10.0;
	ksw->options()->magLimitDrawStarInfo = fNewValue;
	// force redraw
	ksw->skymap->Update();
}

void ViewOpsDialog::newColor( QListBoxItem *item ) {
	QPixmap *temp = new QPixmap( 18, 12 );
  QColor newColor;
	if ( item->text() == tr( "Sky" ) ) {
		newColor = QColorDialog::getColor( QColor( ksw->options()->colorSky ) );
		if ( newColor.isValid() ) {
			ksw->options()->colorSky = newColor.name();
			temp->fill( QColor( ksw->options()->colorSky ) );
		}
  } else if ( item->text() == tr( "Messier Object" ) ) {
		newColor = QColorDialog::getColor( QColor( ksw->options()->colorMess ) );
		if ( newColor.isValid() ) {
			ksw->options()->colorMess = newColor.name();
			temp->fill( QColor( ksw->options()->colorMess ) );
		}
  } else if ( item->text() == tr( "NGC Object" ) ) {
		newColor = QColorDialog::getColor( QColor( ksw->options()->colorNGC ) );
		if ( newColor.isValid() ) {
			ksw->options()->colorNGC = newColor.name();
			temp->fill( QColor( ksw->options()->colorNGC ) );
		}
  } else if ( item->text() == tr( "IC Object" ) ) {
		newColor = QColorDialog::getColor( QColor( ksw->options()->colorIC ) );
		if ( newColor.isValid() ) {
			ksw->options()->colorIC = newColor.name();
			temp->fill( QColor( ksw->options()->colorIC ) );
		}
  } else if ( item->text() == tr( "Object w/ Links" ) ) {
		newColor = QColorDialog::getColor( QColor( ksw->options()->colorHST ) );
		if ( newColor.isValid() ) {
			ksw->options()->colorHST = newColor.name();
			temp->fill( QColor( ksw->options()->colorHST ) );
		}
  } else if ( item->text() == tr( "Milky Way" ) ) {
		newColor = QColorDialog::getColor( QColor( ksw->options()->colorMW ) );
		if ( newColor.isValid() ) {
			ksw->options()->colorMW = newColor.name();
			temp->fill( QColor( ksw->options()->colorMW ) );
		}
  } else if ( item->text() == tr( "Equator" ) ) {
		newColor = QColorDialog::getColor( QColor( ksw->options()->colorEq ) );
		if ( newColor.isValid() ) {
			ksw->options()->colorEq = newColor.name();
			temp->fill( QColor( ksw->options()->colorEq ) );
		}
  } else if ( item->text() == tr( "Ecliptic" ) ) {
		newColor = QColorDialog::getColor( QColor( ksw->options()->colorEcl ) );
		if ( newColor.isValid() ) {
			ksw->options()->colorEcl = newColor.name();
			temp->fill( QColor( ksw->options()->colorEcl ) );
		}
  } else if ( item->text() == tr( "Horizon" ) ) {
		newColor = QColorDialog::getColor( QColor( ksw->options()->colorHorz ) );
		if ( newColor.isValid() ) {
			ksw->options()->colorHorz = newColor.name();
			temp->fill( QColor( ksw->options()->colorHorz ) );
		}
  } else if ( item->text() == tr( "Coordinate Grid" ) ) {
		newColor = QColorDialog::getColor( QColor( ksw->options()->colorGrid ) );
		if ( newColor.isValid() ) {
			ksw->options()->colorGrid = newColor.name();
			temp->fill( QColor( ksw->options()->colorGrid ) );
		}
  } else if ( item->text() == tr( "Star Name" ) ) {
		newColor = QColorDialog::getColor( QColor( ksw->options()->colorSName ) );
		if ( newColor.isValid() ) {
			ksw->options()->colorSName = newColor.name();
			temp->fill( QColor( ksw->options()->colorSName ) );
		}
  } else if ( item->text() == tr( "Constell. Line" ) ) {
		newColor = QColorDialog::getColor( QColor( ksw->options()->colorCLine ) );
		if ( newColor.isValid() ) {
			ksw->options()->colorCLine = newColor.name();
			temp->fill( QColor( ksw->options()->colorCLine ) );
		}
  } else if ( item->text() == tr( "Constell. Name" ) ) {
		newColor = QColorDialog::getColor( QColor( ksw->options()->colorCName ) );
		if ( newColor.isValid() ) {
			ksw->options()->colorCName = newColor.name();
			temp->fill( QColor( ksw->options()->colorCName ) );
		}
	}

	if ( newColor.isValid() )
		ColorPalette->changeItem( *temp, item->text(), ColorPalette->index( item ) );

	delete temp;
	ksw->skymap->Update();
}

void ViewOpsDialog::defaultColors( void ) {
	bool result = setColors( "default.colors" );
	if (!result) {
		QString message = tr( "The specified color scheme file could not be found." );
		QMessageBox::warning( 0, tr( "Could not set color scheme" ), message );
	}
}

void ViewOpsDialog::redColors( void ) {
	bool result = setColors( "night.colors" );
	if (!result) {
		QString message = tr( "The specified color scheme file could not be found." );
		QMessageBox::warning( 0, tr( "Could not set color scheme" ), message );
	}
}

void ViewOpsDialog::chartColors( void ) {
	bool result = setColors( "chart.colors" );
	if (!result) {
		QString message = tr( "The specified color scheme file could not be found." );
		QMessageBox::warning( 0, tr( "Could not set color scheme" ), message );
	}
}

void ViewOpsDialog::slotPreset( int index ) {
	QStringList::Iterator it = PresetFileList.at( index );
	bool result = setColors( *it );
	if (!result) {
		QString message = tr( "The specified color scheme file could not be found." );
		QMessageBox::warning( 0, tr( "Could not set color scheme" ), message );
	}
}

void ViewOpsDialog::slotAddPreset( void ) {
	QFile file;
	QString filename;

	KLineEditDlg getName( tr( "Enter a name for the new scheme:" ), "", this );

	if ( getName.exec() == QDialog::Accepted ) {
		filename = getName.text();
		for( int i=0; i<8; ++i)
			if ( filename.at(i)==' ' ) filename.replace( i, 1, "-" );

		filename = filename.append( ".colors" );
		file.setName( Global::applicationFileName( "kstars", filename ) ); //determine filename in local user KDE directory tree.

		if ( file.exists() || !file.open( IO_ReadWrite | IO_Append ) ) {
			QString message = tr( "Local color scheme file could not be opened.\nScheme cannot be recorded." );
			QMessageBox::warning( 0, tr( "Could not open file" ), message );
		} else {
			QTextStream stream( &file );
			stream << ksw->options()->starColorMode << endl;
			stream << ksw->options()->colorSky << endl;
			stream << ksw->options()->colorMess << endl;
			stream << ksw->options()->colorNGC << endl;
			stream << ksw->options()->colorIC << endl;
			stream << ksw->options()->colorHST << endl;
			stream << ksw->options()->colorSName << endl;
			stream << ksw->options()->colorCName << endl;
			stream << ksw->options()->colorCLine << endl;
			stream << ksw->options()->colorMW << endl;
			stream << ksw->options()->colorEq << endl;
			stream << ksw->options()->colorEcl << endl;
			stream << ksw->options()->colorHorz << endl;
			stream << ksw->options()->colorGrid << endl;
			file.close();
		}

		file.setName( Global::applicationFileName( "kstars", "colors.dat" ) ); //determine filename in local user KDE directory tree.

		if ( !file.open( IO_ReadWrite | IO_Append ) ) {
			QString message = tr( "Local color scheme index file could not be opened.\nScheme cannot be recorded." );
			QMessageBox::warning( 0, tr( "Could not open file" ), message );
		} else {
			QTextStream stream( &file );
			stream << getName.text() << ":" << filename << endl;
			file.close();
		}

		PresetBox->insertItem( getName.text() );
		PresetFileList.append( filename );
	}
}

bool ViewOpsDialog::setColors( QString filename ) {
	QPixmap *temp = new QPixmap( 18, 12 );
	QFile file;
	int i=0;

	if ( !KStarsData::openDataFile( file, filename ) ) {
		file.setName( Global::applicationFileName( "kstars", filename ) ); //try filename in local user KDE directory tree.
		if ( !file.open( IO_ReadOnly ) ) {
			return false;
    }
	}

	QTextStream stream( &file );
	QString line;

	//first line is the star-color mode
  line = stream.readLine();
	int newmode = line.left(1).toInt();
	ksw->options()->starColorMode = newmode;
	if ( ksw->skymap->starColorMode() != newmode )
		ksw->skymap->setStarColorMode( newmode );
	StarColorMode->setCurrentItem( newmode );

	//sky
  line = stream.readLine();
	ksw->options()->colorSky = line.left( 7 );
	temp->fill( QColor( ksw->options()->colorSky ) );
	ColorPalette->changeItem( *temp, ColorPalette->item(i)->text(), i );
	i++;

  //Messier
  line = stream.readLine();
	ksw->options()->colorMess = line.left( 7 );
	temp->fill( QColor( ksw->options()->colorMess ) );
	ColorPalette->changeItem( *temp, ColorPalette->item(i)->text(), i );
	i++;

  //NGC
  line = stream.readLine();
	ksw->options()->colorNGC = line.left( 7 );
	temp->fill( QColor( ksw->options()->colorNGC ) );
	ColorPalette->changeItem( *temp, ColorPalette->item(i)->text(), i );
	i++;

  //IC
  line = stream.readLine();
	ksw->options()->colorIC = line.left( 7 );
	temp->fill( QColor( ksw->options()->colorIC ) );
	ColorPalette->changeItem( *temp, ColorPalette->item(i)->text(), i );
	i++;

  //Links
  line = stream.readLine();
	ksw->options()->colorHST = line.left( 7 );
	temp->fill( QColor( ksw->options()->colorHST ) );
	ColorPalette->changeItem( *temp, ColorPalette->item(i)->text(), i );
	i++;

  //Star Names
  line = stream.readLine();
	ksw->options()->colorSName = line.left( 7 );
	temp->fill( QColor( ksw->options()->colorSName ) );
	ColorPalette->changeItem( *temp, ColorPalette->item(i)->text(), i );
	i++;

  //Constell Names
  line = stream.readLine();
	ksw->options()->colorCName = line.left( 7 );
	temp->fill( QColor( ksw->options()->colorCName ) );
	ColorPalette->changeItem( *temp, ColorPalette->item(i)->text(), i );
	i++;

  //Constell Lines
  line = stream.readLine();
	ksw->options()->colorCLine = line.left( 7 );
	temp->fill( QColor( ksw->options()->colorCLine ) );
	ColorPalette->changeItem( *temp, ColorPalette->item(i)->text(), i );
	i++;

  //Milky Way
  line = stream.readLine();
	ksw->options()->colorMW = line.left( 7 );
	temp->fill( QColor( ksw->options()->colorMW ) );
	ColorPalette->changeItem( *temp, ColorPalette->item(i)->text(), i );
	i++;

  //Equator
  line = stream.readLine();
	ksw->options()->colorEq = line.left( 7 );
	temp->fill( QColor( ksw->options()->colorEq ) );
	ColorPalette->changeItem( *temp, ColorPalette->item(i)->text(), i );
	i++;

  //Ecliptic
  line = stream.readLine();
	ksw->options()->colorEcl = line.left( 7 );
	temp->fill( QColor( ksw->options()->colorEcl ) );
	ColorPalette->changeItem( *temp, ColorPalette->item(i)->text(), i );
	i++;

  //Horizon
  line = stream.readLine();
	ksw->options()->colorHorz = line.left( 7 );
	temp->fill( QColor( ksw->options()->colorHorz ) );
	ColorPalette->changeItem( *temp, ColorPalette->item(i)->text(), i );
	i++;

  //Coordinate Grid
  line = stream.readLine();
	ksw->options()->colorGrid = line.left( 7 );
	temp->fill( QColor( ksw->options()->colorGrid ) );
	ColorPalette->changeItem( *temp, ColorPalette->item(i)->text(), i );
  i++;

	ksw->skymap->Update();
	return true;
}

void ViewOpsDialog::updateDisplay( void ) {
	ksw->options()->drawBSC = showBSC->isChecked();
	ksw->options()->drawMessier = showMessier->isChecked();
	ksw->options()->drawMessImages = showMessImages->isChecked();
	ksw->options()->drawNGC = showNGC->isChecked();
	ksw->options()->drawIC = showIC->isChecked();
	ksw->options()->drawSun = showSun->isChecked();
	ksw->options()->drawMoon = showMoon->isChecked();
	ksw->options()->drawMercury = showMercury->isChecked();
	ksw->options()->drawVenus = showVenus->isChecked();
	ksw->options()->drawMars = showMars->isChecked();
	ksw->options()->drawJupiter = showJupiter->isChecked();
	ksw->options()->drawSaturn = showSaturn->isChecked();
	ksw->options()->drawUranus = showUranus->isChecked();
	ksw->options()->drawNeptune = showNeptune->isChecked();
	ksw->options()->drawPluto = showPluto->isChecked();
	ksw->options()->drawConstellLines = showConstellLines->isChecked();
	ksw->options()->drawConstellNames = showConstellNames->isChecked();
	ksw->options()->useLatinConstellNames = useLatinConstellNames->isChecked();
	ksw->options()->useLocalConstellNames = useLocalConstellNames->isChecked();
	ksw->options()->useAbbrevConstellNames = useAbbrevConstellNames->isChecked();
	ConstellOptions->setEnabled( showConstellNames->isChecked() );
	ksw->options()->drawMilkyWay = showMilkyWay->isChecked();
	ksw->options()->fillMilkyWay = showMilkyWayFilled->isChecked();
	showMilkyWayFilled->setEnabled( showMilkyWay->isChecked() );
	ksw->options()->drawGrid = showGrid->isChecked();
	ksw->options()->drawEquator = showEquator->isChecked();
	ksw->options()->drawEcliptic = showEcliptic->isChecked();
	ksw->options()->drawHorizon = showHorizon->isChecked();
	ksw->options()->drawGround = showGround->isChecked();
	ksw->options()->drawStarName 		 = showStarNames->isChecked();
	ksw->options()->drawStarMagnitude = showStarMagnitude->isChecked();

	ksw->skymap->Update();
}

void ViewOpsDialog::changeCoordSys( void ) {
	ksw->options()->useAltAz = AltAzRadio->isChecked();
	showGround->setEnabled( AltAzRadio->isChecked() );

	if ( EquatRadio->isChecked() ) {
		showGround->setChecked( false );
	}
	ksw->options()->drawGround = showGround->isChecked();
	ksw->skymap->Update();
}

void ViewOpsDialog::changeStarColorIntensity( int newValue ) {
	ksw->skymap->setStarColorIntensity( newValue );
	ksw->options()->starColorIntensity = ksw->skymap->starColorIntensity();
	ksw->skymap->Update();
}

void ViewOpsDialog::changeStarColorMode( int newValue ) {
	ksw->skymap->setStarColorMode( newValue );
	ksw->options()->starColorMode = ksw->skymap->starColorMode();
	if (newValue) IntensityBox->setEnabled( false );
	else IntensityBox->setEnabled( true );
	ksw->skymap->Update();
}

void ViewOpsDialog::markPlanets( void ) {
	showSun->setChecked( true );
	showMoon->setChecked( true );
	showMercury->setChecked( true );
	showVenus->setChecked( true );
	showMars->setChecked( true );
	showJupiter->setChecked( true );
	showSaturn->setChecked( true );
	showUranus->setChecked( true );
	showNeptune->setChecked( true );
	showPluto->setChecked( true );
	updateDisplay();
}

void ViewOpsDialog::unMarkPlanets( void ) {
	showSun->setChecked( false );
	showMoon->setChecked( false );
	showMercury->setChecked( false );
	showVenus->setChecked( false );
	showMars->setChecked( false );
	showJupiter->setChecked( false );
	showSaturn->setChecked( false );
	showUranus->setChecked( false );
	showNeptune->setChecked( false );
	showPluto->setChecked( false );
	updateDisplay();
}

